#ifndef CONNECTIONS_H
#define CONNECTIONS_H

#include "qobject.h"
#include <qiconset.h>

#include "Statistic.h"
#include "Address.h"

class QtellaSub;
class QListViewItem;
class QWidget;
class QTimer;
class QDateTime;
class QSocket;
class Servent;
class QueryHitEntry;
class SearchWidget;
class Search;
class Message;

#include <vector>
#include <string>
#include <ctime>
#include <list>

#if !defined(_WIN32)
using namespace std;
#endif

class Connections : QObject
{
  Q_OBJECT

 public:
  Connections(QtellaSub *parent = 0);
  ~Connections();

  static Connections* getInstance();

  void               start();
  void               search(std::string query, SearchWidget *sw);
  void               checkServants();
  void               checkMessages(Servent* servant);
  void               disconnect();
  std::string        createID();
  static std::string toHex(std::string data);
  void               addServant(Address address, QSocket* socket = NULL, const std::string user_agent = "", const std::string handshake = "");
  void               sendAll(std::string data);
  
  std::vector<Search*>     _search;
  std::vector<Servent*>    _servants;

  std::string         _id;
  QtellaSub*          _parent;
  Statistic           _statistic;
  Address             _address;

  const bool         incomingSlotsFree();
  const void         closeSocket( QSocket* );

 protected:
  static Connections*        _this;

  int                 _max_servants;
  int                 _max_hosts;
  QTimer*             _statistic_timer;
  QDateTime*          _uptime;
  QTimer*             _timer;

  std::list<std::pair<time_t, QSocket*> > _sockets_to_close;

 public slots:
  void     check();
  void     slotRemoveHost();
  void     slotAddHost();
  void     slotMaxServents(int value);
  void     slotListClear();
  void     slotListRemove();
  void     slotListConnect();
  void     slotUpdateStatistic();
  void     slotBlockHost();
  void     slotShowHandshake();
};

#endif
