#include "HostCacheConnection.h"
#include "HostCache.h"
#include "QtellaSub.h"
#include "Message.h"
#include "Ping.h"
#include "Pong.h"
#include <qfile.h>
#include <qsocketdevice.h>
#include <qdatetime.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <string>

QMutex HostCacheConnection::_mutex;

HostCacheConnection::HostCacheConnection(int socket, HostCache* parent, std::list<Pong>& ponglist)
  : _parent(parent), _socket(socket), _ponglist(ponglist)
{
}

void HostCacheConnection::run()
{
  QSocketDevice* s = new QSocketDevice(_socket, QSocketDevice::Stream);

  if(! ((QtellaSub*)(_parent->_parent))->ui_checkbox_enablehostcache->isChecked() ) 
    {
      s->close();
      delete s;
      return;
    }

  std::string connectstr("GNUTELLA CONNECT/0.4\n\n");
  int timeout = 0;

  while( (s->bytesAvailable() < connectstr.size()) && (timeout < 50) )
    {
      msleep(10);
      timeout++;
    }

  std::string logging;
  logging += std::string(QDate::currentDate().toString());
  logging += " " + std::string(QTime::currentTime().toString());
  logging += " Connection from ";
  logging += std::string(s->peerAddress().toString());

  char buffer[50];

  if(timeout < 50)
    {
      s->readBlock(buffer, connectstr.size());

      if( std::string(buffer, connectstr.size()) == connectstr )
	{
	  logging += " ok";
	  s->writeBlock(std::string("GNUTELLA OK\n\n").data(), 13);

	  timeout = 0;
	  while( (s->bytesAvailable() < 23) && (timeout < 200) )
	    {
	      msleep(10);
	      timeout++;
	    }

	  if(timeout < 100)
	    {
	      logging += " ok";
	      s->readBlock(buffer, 23);

	      std::string     msg(buffer, 23);
	      Message         m(msg);
	      Ping*           p = m.getPing();

	      std::list<Pong>::iterator     pos;

	      for( pos = _ponglist.begin(); pos != _ponglist.end(); ++pos)
		{
		  pos->_id = p->_id;
		  s->writeBlock( (*pos)._data.c_str(), (*pos)._data.size() );
		}
	      
	      s->flush();
	      this->msleep(1500);
	      s->close();

	      delete p;
	    }
	  else
	    logging += " timeout";
	}
      else
	logging += " error";
    }
  else
    logging += " timeout";

  _mutex.lock();

  QListBox* l = ((QtellaSub*)(_parent->_parent))->ui_listbox_hostcachelogging;

  if( ((QtellaSub*)(_parent->_parent))->ui_checkbox_enablelogging->isChecked() )
    {

      if( l->count() > 200 ) l->removeItem(0);

      l->insertItem(logging.c_str());
      l->setBottomItem(l->count()-1);

      if(! std::string( ((QtellaSub*)(_parent->_parent))->ui_lineedit_hostcachelogfile->text() ).empty() )
	{
	  QFile logfile( ((QtellaSub*)(_parent->_parent))->ui_lineedit_hostcachelogfile->text() );
	  if(logfile.open(IO_Raw|IO_Append|IO_ReadWrite))
	    {
	      logfile.writeBlock( (logging+"\n").c_str(), logging.size()+1 );
	      logfile.close();
	    }
	}

    }

  _mutex.unlock();

  delete s;
}
