#include "InterruptedDownloads.h"
#include "InterruptedFile.h"
#include "QtellaSub.h"
#include "Logging.h"
#include "StringManipulation.h"
#include "SearchDialog.h"
#include "InterruptedFileSearch.h"
#include "Icon.h"
#include "DownloadManager.h"
#include "ResumeHost.h"
#include "InterruptedViewItem.h"

#include <qlistview.h>
#include <qlistview.h>
#include <qdir.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qlineedit.h>

#include <strstream>
#include <iostream>
#include <iomanip>

InterruptedDownloads* InterruptedDownloads::_this = NULL;

InterruptedDownloads::InterruptedDownloads(QtellaSub* parent, std::string directory)
  : _parent(parent)
{
  _this = this;

  if(!directory.empty())
    if(directory[directory.size()-1] != '/') directory += "/";
  _directory = directory;
  
  _context_menu = new QPopupMenu(parent, tr("Interrupted Downloads"));
  _context_menu->insertItem( tr("Search for file"), this, SLOT(searchSelectedFiles()), CTRL+Key_C);
  _context_menu->insertItem( tr("Resume"), this, SLOT(resumeSelectedFiles()), CTRL+Key_S);
  //_context_menu->insertItem( tr("Resume from host"), this, SLOT(resumeFromHost()), CTRL+Key_H);
  _context_menu->insertSeparator();
  _context_menu->insertItem( tr("Abort Download"), this, SLOT(abortSelectedFiles()), CTRL+Key_A);
  _context_menu->insertItem( tr("Delete from disc"), this, SLOT(deleteSelectedFiles()), CTRL+Key_Delete);
  _context_menu->insertSeparator();
  _context_menu->insertItem( tr("Launch"), parent, SLOT(slotLaunchInterruptedFile()), CTRL+Key_L);
  

  update();
}


InterruptedDownloads* InterruptedDownloads::getInstance()
{
  return _this;
}


void InterruptedDownloads::update()
{
  // find all interrupted files in the directory
  std::map<std::string, InterruptedFile> files;
  QDir d(_directory.c_str(), "*", QDir::Name|QDir::IgnoreCase, QDir::Files);
  for(int i = 0; i < d.count(); ++i)
    {
      InterruptedFile file(_directory + "/" + std::string(d[i]));
      if( ! file.isValid() ) continue;

      files.insert( std::make_pair(file.completeName(), file) );
    }


  // delete all entries from the listview which do not exist in the directory
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  while(pos != _file_map.end())
    {
      if( files.find(pos->second.completeName()) == files.end() ) 
	{
	  deleteItem(pos->first);
	  _file_map.erase(pos++);
	}
      else
	++pos;
    }


  // add all files from the directory which do not exist in the listview
  std::map<std::string, InterruptedFile>::iterator file_pos = files.begin();
  
  while( file_pos != files.end() )
    {
      for( pos = _file_map.begin(); pos != _file_map.end(); ++pos )
	if( pos->second.completeName() == file_pos->second.completeName() ) 
	  {
	    // update progress
	    pos->first->setText(3, pos->second.strprogress().c_str());
	    break;
	  }

      if( pos == _file_map.end() )
	{
	  InterruptedViewItem* item;

	  std::map<InterruptedViewItem*, InterruptedFile>::iterator p;

	  QListViewItem* parent_item = NULL;

	  for( p = _file_map.begin(); p != _file_map.end(); ++p)
	    if( p->second.size() == file_pos->second.size() )
	      {
		parent_item = p->first;
		if( p->first->firstChild() )
		  {
		    parent_item = p->first;
		    break;
		  }
	      }

	  if( parent_item )
	    {
	      item = new InterruptedViewItem( parent_item );
	      _parent->ui_listview_interrupt->setOpen( parent_item, true );
	    }
	  else
	    item = new InterruptedViewItem(_parent->ui_listview_interrupt);

	  
	  item->setText(0, file_pos->second.filename().c_str());
	  item->setText(1, file_pos->second.ext().c_str());
	  item->setText(2, file_pos->second.strsize().c_str());
	  item->setText(3, file_pos->second.strprogress().c_str());
	  item->setText(4, file_pos->second.ip().c_str());
	  
	  _file_map.insert( std::make_pair(item, file_pos->second) );
	}

      ++file_pos;
    }
}

void InterruptedDownloads::getSelectedFiles(std::vector<InterruptedFile>& files)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->first->isSelected()) files.push_back(pos->second);
}

void InterruptedDownloads::getFiles(std::vector<InterruptedFile>& files, const bool force_update)
{
  if( force_update )
    update();

  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;
  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos) files.push_back(pos->second);
}

void InterruptedDownloads::deleteSelectedFiles()
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); )
    if(pos->first->isSelected())
      {
	if( pos->second.getColored() )
	  {
	    std::map<InterruptedViewItem*, QueryHitEntry>::iterator p = _qhe.find(pos->first);
	    
	    if(p != _qhe.end())
	      QtellaSub::getInstance()->download_manager->deleteDownload(p->second);
	  }
	
	if(!QFile::remove(pos->second.completeName().c_str()))
	  {
	    std::string mess = std::string(tr("Could not remove file"));
	    Logging::getInstance()->log(Logging::Error, mess + " [" + pos->second.completeName() + "]");
	  }
	deleteItem(pos->first);
	_file_map.erase(pos++);
      }
    else ++pos;
}

void InterruptedDownloads::searchSelectedFiles()
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  
  while(pos != _file_map.end())
    {
      if( !pos->first->isSelected() ) { ++pos; continue; }

      std::vector<std::string> v;
      StringManipulation sm(pos->second.filename());
      sm.extractWords(v);

      std::vector<std::string> search_words;
      int i;
      for(i = 0; i < v.size(); ++i)
	if(search_words.size() < 3) search_words.push_back(v[i]);
	else
	  {
	    int j = 0;
	    for(int k = 0; k < search_words.size(); ++k)
	      if(search_words[k].size() < search_words[j].size()) j = k;

	    if(search_words[j].size() < v[i].size()) search_words[j] = v[i];
	  }

      std::string search;
      for(i = 0; i < search_words.size(); ++i) search += search_words[i] + " ";
      if(!search.empty()) search = search.substr(0, search.size()-1);

      SearchDialog d(0, 0, true);
      d.setQuery(search);
      if(d.exec() == SearchDialog::Accepted) _parent->search(d.getQuery());

      ++pos;
    }
}

void InterruptedDownloads::resumeSelectedFiles()
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.begin();
  
  while(pos != _file_map.end())
    {
      if( !pos->first->isSelected() ) { ++pos; continue; }

      if( pos->second.getColored() )
	{
	  QMessageBox::information(0, tr("Information"), 
				   tr("An interrupted file can be resumed only one time."), QMessageBox::Ok);
	  ++pos;
	  continue;
	}

      InterruptedFileSearch d(0, 0, true);
      d.init(pos->second.size());
      if(d.exec() == InterruptedFileSearch::Accepted)
	{
	  QueryHitEntry q = *d.getSelectedQueryHit();

	  InterruptedViewItem* item = (InterruptedViewItem*)q.item;
	  q.item = NULL; // do not destroy item when contructor is called

	  _qhe.insert( std::make_pair(pos->first, q) );
	  _parent->download_manager->addDownload(q, item, pos->second.filename(), pos->first);
	}

      ++pos;
    }
}

void InterruptedDownloads::newQueryHit(unsigned long fsize)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->second.size() == fsize) 
      {
	pos->second.incFound();
	updateBall(pos->first);
      }
}

void InterruptedDownloads::removeQueryHit(unsigned long fsize)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if(pos->second.size() == fsize) 
      {
	pos->second.decFound();
	updateBall(pos->first);
      }
}

void InterruptedDownloads::setYellow(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getYellowBall()));
}

void InterruptedDownloads::setRed(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getRedBall()));
}

void InterruptedDownloads::setGreen(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getGreenBall()));
}

void InterruptedDownloads::setSmile(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getSmile()));
}

void InterruptedDownloads::setGreenSmile(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.setColored();
  item->setPixmap(0, *(Icon::getInstance()->getGreenSmile()));
}

void InterruptedDownloads::updateBall(InterruptedViewItem *item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;

  if( pos->second.getFound() > 0 )
    if( pos->second.getColored() == 0 ) item->setPixmap(0, *(Icon::getInstance()->getGreyBall()));

  if( pos->second.getFound() == 0)
    if( pos->second.getColored() == 0) item->setPixmap(0, QPixmap());
}

void InterruptedDownloads::clearColored(InterruptedViewItem* item)
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.find(item);
  if(pos == _file_map.end()) return;
  pos->second.clearColored();
  updateBall(item);
}

void InterruptedDownloads::slotShowContextMenu(InterruptedViewItem* item, const QPoint& p, int i)
{
  _context_menu->popup(p);
}

void InterruptedDownloads::abortSelectedFiles()
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos;

  for(pos = _file_map.begin(); pos != _file_map.end(); ++pos)
    if( (pos->first->isSelected()) && (pos->second.getColored()) )
      {
	std::map<InterruptedViewItem*, QueryHitEntry>::iterator p = _qhe.find(pos->first);
	
	if(p != _qhe.end())
	  QtellaSub::getInstance()->download_manager->abortDownload(p->second);
      }
}

void InterruptedDownloads::convertFilenames()
{
  int er = 0;
  int ok = 0;
  int no = 0;

  QDir d(_directory.c_str(), "*", QDir::Name|QDir::IgnoreCase, QDir::Files);

  for(int i = 0; i < d.count(); ++i)
    {
      InterruptedFile ifile(_directory + "/" + std::string(d[i]));
      
      if(!ifile.isValid()) continue;

      std::string n = ifile.completeName();
      std::string::size_type idx = n.find(".part_");

      for(int j = 0; j < 4; ++j) idx = n.find("_", idx + 1);
      idx = n.find_first_not_of("0123456789", idx + 1);

      if( n[idx] == '_' )
	++no;
      else
	{
	  std::strstream str;
	  str << n.substr(0, idx) << "_0-" << (ifile.size() - 1) << n.substr(idx) << std::ends;
	  
	  QDir dir;
	  if( dir.rename(n.c_str(), str.str()) )
	    ++ok;
	  else
	    ++er;
	}
    }

  std::strstream str;
  str << ok << " " << (const char*)tr("files have been converted.") << std::endl
      << er << " " << (const char*)tr("errors.") << std::endl
      << no << " " << (const char*)tr("not converted.") << std::ends;
  QMessageBox::information(0, tr("Information"), str.str(), QMessageBox::Ok, 0);
  str.freeze(false);

  update();
}


void InterruptedDownloads::resumeFromHost()
{
  std::map<InterruptedViewItem*, InterruptedFile>::iterator pos = _file_map.begin();

  while(pos != _file_map.end())
    if( pos->first->isSelected() )
      {
	if( pos->second.getColored() )
	  {
	    QMessageBox::information(0, tr("Information"), 
				     tr("An interrupted file can be resumed only one time."), QMessageBox::Ok);
	    ++pos;
	    continue;
	  }

	ResumeHost rh(0, 0, true);
	rh.ui_lineedit_ip->setText(pos->second.host().c_str());
  
	if( rh.exec() == ResumeHost::Accepted )
	  {
	    //QueryHitEntry q // geht nicht ??
	  }
	++pos;
      }
    else ++pos;

//       InterruptedFileSearch d(0, 0, true);
//       d.init(pos->second.size());
//       if(d.exec() == InterruptedFileSearch::Accepted)
// 	{
// 	  QueryHitEntry q = *d.getSelectedQueryHit();

// 	  InterruptedViewItem* item = q.item;
// 	  q.item = NULL; // do not destroy item when contructor is called

// 	  _qhe.insert( make_pair(pos->first, q) );
// 	  _parent->download_manager->addDownload(q, item, pos->second.filename(), pos->first);
// 	}

//       ++pos;



}


void InterruptedDownloads::deleteItem( InterruptedViewItem* item )
{
  if( item->firstChild() )
    {
      QListViewItem* first = item->firstChild();
      first->moveItem(item);

      QListViewItem* i;
      while( ( i = item->firstChild() ) )
	{
	  item->takeItem(i);
	  first->insertItem(i);
	}
    }

  delete item;
}
