#include "../include/MergeDialog.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qframe.h>

#include <strstream>
#include <iomanip>

/* 
 *  Constructs a MergeDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
MergeDialog::MergeDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : BMergeDialog( parent, name, modal, fl )
{
  _fsize = 0;

  connect(ui_spinbox_start, SIGNAL(valueChanged(int)), this, SLOT(slotChangeValue()));
  connect(ui_spinbox_end, SIGNAL(valueChanged(int)), this, SLOT(slotChangeValue()));
}

/*  
 *  Destroys the object and frees any allocated resources
 */
MergeDialog::~MergeDialog()
{
    // no need to delete child widgets, Qt does it all for us
}


void MergeDialog::setFileSize(unsigned int size)
{
  _fsize = size;

  ui_spinbox_end->setMaxValue(size - 1);
  ui_spinbox_end->setValue(size - 1);
  ui_spinbox_start->setMaxValue(size);
  ui_spinbox_start->setValue(0);


  std::strstream filesize;
  filesize.setf(std::ios::fixed);
  filesize << std::setprecision(2) 
	   << static_cast<double>(size) / 1024 / 1024 << "MB" << std::ends;
  ui_textlabel_size->setText(filesize.str());
  filesize.freeze(false);


  _img = new QImage (ui_frame_read->width(), ui_frame_read->height(), 32, 16);
  _img->fill(qRgb(0xff, 0xff, 0xff));

  QPixmap p;
  p.convertFromImage(*_img);
  ui_frame_read->setBackgroundPixmap(p);

  // set filename
  if( _qhe.filename.size() > 50 )
    ui_textlabel_filename->setText( (_qhe.filename.substr(0, 50) + "...").c_str() );
  else
    ui_textlabel_filename->setText( _qhe.filename.c_str() );

  // calculation missing segments
}


void MergeDialog::updateSelected()
{
  if( ! _fsize ) return;

  _img_sel = new QImage (ui_frame_sel->width(), ui_frame_sel->height(), 32, 16);
  _img_sel->fill(qRgb(0xff, 0xff, 0xff));

  int w = _img_sel->width();
  int s = ui_spinbox_start->value();
  int e = ui_spinbox_end->value();

  int posx1 = w * s / _fsize;
  int posx2 = w * e / _fsize;

  for(int i = posx1; i < posx2; ++i) 
    for(int y = 1; y < _img_sel->height(); ++y) _img_sel->setPixel(i, y, qRgb(0xff, 0, 0));

  QPixmap p;
  p.convertFromImage(*_img_sel);
  ui_frame_sel->setBackgroundPixmap(p);
  ui_frame_sel->update();
}


void MergeDialog::slotChangeValue()
{
  updateSelected();
}
