#include "QtellaSub.h"
#include "Gnutella.h"

#include "Connections.h"
#include "StringManipulation.h"
#include "QueryHitEntry.h"
#include "SearchWidget.h"
#include "Search.h"
#include "SharedFile.h"
#include "UploadManager.h"
#include "DownloadManager.h"
#include "IncomingConnect.h"
#include "SharedViewItem.h"
#include "AddApplication.h"
#include "Download.h"
#include "HostCache.h"
#include "DownloadViewItem.h"
#include "InterruptedDownloads.h"
#include "IconTabWidget.h"
#include "Icon.h"
#include "SearchSharedFiles.h"
#include "Interface.h"
#include "DeleteDialog.h"
#include "RenameFile.h"
#include "BlockHost.h"
#include "SearchViewItem.h"

#include <fstream>
#include <algorithm>
#include <iomanip>
#include <strstream>
#include <iostream>
#include <cctype>
#if !defined(_WIN32)
#include <unistd.h>
#endif

#include <qrect.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlabel.h>
#include <qfile.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qfiledialog.h>
#include <qcheckbox.h>
#include <qthread.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qiconset.h>
#include <qpalette.h>
#include <qmessagebox.h>
#include <qwidget.h>

#include <signal.h>

#if QT_VERSION >= 300
#include <qstylefactory.h>
#endif

#ifdef KDE
#include "kmainwindow.h"
#include "ksystemtray.h"  /* system tray patch */
#include <krun.h>
#include <kopenwith.h>
#endif

#include <qtoolbar.h>
#include <qstringlist.h>

#include "qtella_mini_xpm.h" /* system tray patch */

#undef _DEBUG

QtellaSub* QtellaSub::_instance = NULL;

/* 
 *  Constructs a QtellaSub which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
QtellaSub::QtellaSub( QApplication* parent,  const char* name, WFlags fl )
  : Qtella( 0, name, fl ), _binary(name)
{
#ifdef KDE
  KSystemTray *tray = new KSystemTray(this,"qtella");
  tray->setPixmap(QPixmap((const char **)qtella_mini_xpm));
  tray->setAlignment(Qt::AlignCenter);
  tray->show();
#endif

  int i;
  int j;

  _style_s = "default";
  _prev_version = "";
  _search_column_size = 350;
  _instance = this;
  _current_font = qApp->font();
  _incoming_allowed = false;

  #ifdef KDE
    _style = new KPopupMenu(this);
  #endif

  #ifdef QT
    _style = new QPopupMenu(this);
  #endif

#if QT_VERSION < 300
  _style->insertItem("&Platinum", this, SLOT(slotStylePlatinum()), 0, 0);
  _style->insertItem("&SGI", this, SLOT(slotStyleSGI()), 0, 1);
  _style->insertItem("&Windows", this, SLOT(slotStyleWindows()), 0, 2);
  _style->insertItem("&Motif", this, SLOT(slotStyleMotif()), 0, 3);
  _style->insertItem("Motif&Plus", this, SLOT(slotStyleMotifPlus()), 0, 4);
  _style->insertItem("&CDE", this, SLOT(slotStyleCDE()), 0, 5);
  #ifdef KDE
  _style->insertItem("&KDE", this, SLOT(slotStyleKDE()), 0, 6);
  #endif
#else
  QStringList styles = QStyleFactory::keys();
  for(QStringList::ConstIterator it=styles.begin(); it!=styles.end(); it++) 
    {
      int id=_style->insertItem("&" + *it, this, SLOT(slotStyle(int)));
      _style->setItemParameter(id, id);
    }
#endif

  _style->insertSeparator();
  _style->insertItem( tr("&Font"), this, SLOT(slotChangeFont()));

  _style->setCheckable(true);

#ifdef KDE
  _help = new KPopupMenu(this);
#endif

#ifdef QT
  _help = new QPopupMenu(this);
#endif

  _help->insertItem( tr("&Info"), this, SLOT(slotInfo()));


#ifdef KDE
  _file = new KPopupMenu(this);
#endif

#ifdef QT
  _file = new QPopupMenu(this);
#endif

  _file->insertItem( tr("&Connect"), this, SLOT(slotConnect()));
  _file->insertItem( tr("&Disconnect"), this, SLOT(slotDisconnect()));
  _file->insertSeparator();
  _file->insertItem( tr("Convert &Filenames"), this, SLOT(slotConvertFilenames()));
  _file->insertSeparator();
  _file->insertItem( tr("E&xit"), parent, SLOT(quit()));

#ifdef KDE
  _view = new KPopupMenu(this);
#endif

#ifdef QT
  _view = new QPopupMenu(this);
#endif

  _view->insertItem( tr("Connections"), 1);
  _view->insertItem( tr("Search"), 2);
  _view->insertItem( tr("Downloads"), 3);
  _view->insertItem( tr("Uploads"), 4);
  _view->insertItem( tr("Shared Files"), 5);
  _view->insertItem( tr("Logs"), 6);
  _view->insertItem( tr("Statistics"), 7);
  _view->insertItem( tr("Hostcache"), 8);
  _view->insertItem( tr("Interrupted Downloads"), 9);
  _view->insertItem( tr("Configuration"), 10);

  connect(_view, SIGNAL(activated(int)), SLOT(slotViewChanged(int)));
  _view->setCheckable(true);

  for(i = 1; i < 11; ++i)
    _view->setItemChecked(i, true);
  
#ifdef KDE
  menu = new KMenuBar(this);
#endif

#ifdef QT
  menu = new QMenuBar(this);
#endif

  menu->insertItem( tr("&File"), _file);
  menu->insertItem( tr("&Style"), _style);
  menu->insertItem( tr("&View"), _view);
  menu->insertSeparator();
  menu->insertItem( tr("&Help"), _help);


  _palette = qApp->palette();
  _menu_palette = menu->palette();


  
  std::strstream str;
  str << "Qtella - " << VERSION << " - http://www.qtella.net" << std::ends;
  std::string caption = str.str();
  str.freeze(false);
  setCaption( caption.c_str() );

  connections = new Connections(this);

  timer_connection = new QTimer(this);
  connect(timer_connection, SIGNAL(timeout()), this, SLOT(check_connections()));

  connect((QObject*)ui_pushbutton_remove, SIGNAL(clicked()), (QObject*)connections, SLOT(slotRemoveHost()));
  connect((QObject*)ui_spinbox_hosts, SIGNAL(valueChanged(int)), (QObject*)connections, SLOT(slotMaxServents(int)));
  connect((QObject*)ui_pushbutton_listclear, SIGNAL(clicked()), (QObject*)connections, SLOT(slotListClear()));
  connect((QObject*)ui_pushbutton_listconnect, SIGNAL(clicked()), (QObject*)connections, SLOT(slotListConnect()));
  connect((QObject*)ui_pushbutton_listremove, SIGNAL(clicked()), (QObject*)connections, SLOT(slotListRemove()));


  ui_listview_download->setColumnWidthMode(0, QListView::Manual);  // filename
  ui_listview_download->setColumnWidthMode(1, QListView::Maximum); // status
  ui_listview_download->setColumnWidthMode(2, QListView::Maximum); // size
  ui_listview_download->setColumnWidthMode(3, QListView::Maximum); // progress 
  ui_listview_download->setColumnWidthMode(4, QListView::Maximum); // host
  ui_listview_download->setColumnWidthMode(5, QListView::Maximum); // bandwidth
  ui_listview_download->setColumnWidthMode(6, QListView::Maximum); // Time
  ui_listview_download->setColumnWidthMode(7, QListView::Maximum); // Client
  ui_listview_download->setColumnWidth(0, 16*10);
  ui_listview_download->setColumnAlignment(0, AlignLeft);
  ui_listview_download->setColumnAlignment(1, AlignLeft);
  ui_listview_download->setColumnAlignment(2, AlignRight);
  ui_listview_download->setColumnAlignment(3, AlignRight);
  ui_listview_download->setColumnAlignment(4, AlignRight);
  ui_listview_download->setColumnAlignment(5, AlignRight);
  ui_listview_download->setColumnAlignment(6, AlignRight);
  ui_listview_download->setColumnAlignment(7, AlignLeft);
  ui_listview_download->setSorting(0);
  ui_listview_download->setColumnWidth(0, 30*10);
  ui_listview_download->setRootIsDecorated(true);

  ui_listview_connections->setColumnWidthMode(0, QListView::Maximum); // host
  ui_listview_connections->setColumnWidthMode(1, QListView::Maximum); // status
  ui_listview_connections->setColumnWidthMode(2, QListView::Maximum); // messages (i/o)
  ui_listview_connections->setColumnWidthMode(3, QListView::Maximum); // bandwidth
  ui_listview_connections->setColumnWidthMode(4, QListView::Maximum); // uptime
  ui_listview_connections->setColumnWidthMode(5, QListView::Maximum); // protocol
  ui_listview_connections->setColumnWidthMode(6, QListView::Maximum); // user agent
  ui_listview_connections->setColumnAlignment(0, AlignLeft);
  ui_listview_connections->setColumnAlignment(1, AlignLeft);
  ui_listview_connections->setColumnAlignment(2, AlignHCenter);
  ui_listview_connections->setColumnAlignment(3, AlignHCenter);
  ui_listview_connections->setColumnAlignment(4, AlignHCenter);
  ui_listview_connections->setColumnAlignment(5, AlignRight);
  ui_listview_connections->setColumnAlignment(6, AlignHCenter);
  ui_listview_connections->setSorting(-1);

  ui_listview_shared->setColumnWidthMode(0, QListView::Manual);  // filename
  ui_listview_shared->setColumnWidthMode(1, QListView::Maximum); // uploads
  ui_listview_shared->setColumnWidthMode(2, QListView::Maximum); // requests
  ui_listview_shared->setColumnWidthMode(3, QListView::Maximum); // size
  ui_listview_shared->setColumnAlignment(0, AlignLeft);
  ui_listview_shared->setColumnWidth(0, 46*10);
  ui_listview_shared->setColumnWidth(1, 5*10);
  ui_listview_shared->setColumnWidth(2, 5*10);
  ui_listview_shared->setColumnWidth(3, 10*10);
  ui_listview_shared->setSorting(0);

  ui_listview_uploads->setColumnWidthMode(0, QListView::Manual);  // filename
  ui_listview_uploads->setColumnWidthMode(1, QListView::Maximum); // status
  ui_listview_uploads->setColumnWidthMode(2, QListView::Maximum); // host
  ui_listview_uploads->setColumnWidthMode(3, QListView::Maximum); // progress
  ui_listview_uploads->setColumnWidthMode(4, QListView::Maximum); // bandwidth
  ui_listview_uploads->setColumnWidthMode(5, QListView::Maximum); // time
  ui_listview_uploads->setColumnAlignment(0, AlignLeft);
  ui_listview_uploads->setColumnWidth(0, 29*10);
  ui_listview_uploads->setColumnWidth(1, 10*10);
  ui_listview_uploads->setColumnWidth(2, 10*10);
  ui_listview_uploads->setColumnWidth(5, 7*10);

  ui_listview_interrupt->setColumnWidthMode(0, QListView::Manual);  // filename
  ui_listview_interrupt->setColumnWidthMode(1, QListView::Maximum); // type
  ui_listview_interrupt->setColumnWidthMode(2, QListView::Maximum); // size
  ui_listview_interrupt->setColumnWidthMode(3, QListView::Maximum); // progress
  ui_listview_interrupt->setColumnWidthMode(4, QListView::Maximum); // host
  ui_listview_interrupt->setColumnAlignment(0, AlignLeft);
  ui_listview_interrupt->setColumnWidth(0, 30*10);
  ui_listview_interrupt->setColumnWidth(1, 7*10);
  ui_listview_interrupt->setColumnWidth(2, 7*10);
  ui_listview_interrupt->setColumnWidth(3, 7*10);
  ui_listview_interrupt->setColumnWidth(4, 10*10);
  ui_listview_interrupt->setRootIsDecorated(true);

  ui_listview_apps->setColumnWidthMode(0, QListView::Maximum);
  ui_listview_apps->setColumnWidthMode(1, QListView::Maximum);
  ui_listview_apps->setColumnWidth(0, 10*10);
  ui_listview_apps->setColumnWidth(1, 10*10);

  // initialise extensions for images, videos and music
  ui_lineedit_images->setText(IMAGES);
  ui_lineedit_video->setText(VIDEOS);
  ui_lineedit_audio->setText(MUSIC);

  _hostcache = NULL;

  // read configuration
  readConfiguration();
  
  // set style settings
#if QT_VERSION < 300
  if( _style_s == "PLATINUM" ) slotStylePlatinum();
  if( _style_s == "SGI" ) slotStyleSGI();
  if( _style_s == "WINDOWS" ) slotStyleWindows();
  if( _style_s == "MOTIF" ) slotStyleMotif();
  if( _style_s == "MOTIFPLUS" ) slotStyleMotifPlus();
  if( _style_s == "CDE" ) slotStyleCDE();
#endif

  // read merge.dat
  //_merge_downloads = new MergeDownloads();

  // get interface address
  _ip_update_timer = new QTimer(this);
  connect(_ip_update_timer, SIGNAL(timeout()), this, SLOT(slotIPUpdate()));
  connect(ui_pushbutton_updateip, SIGNAL(pressed()), SLOT(slotIPUpdate()));
  _ip_update_timer->start(ui_spinbox_updateinterval->value(), 10000);

  Interface inf;
  std::vector<std::string> if_vect;
  if_vect.push_back( std::string( tr("automatic", "network interface") ) );
  if_vect.push_back( std::string( tr("manual", "network interface") ) );
  if_vect.push_back( std::string( tr("file", "network interface") ) );
#if !defined(_WIN32)
  if_vect.push_back( std::string( tr("hostname", "network interface") ) );
#endif

  std::string if_default[] = {"eth0", "ppp0", "ippp0"};
  inf.getIfList(if_vect);


  for(j = 0; j < if_vect.size(); ++j) ui_combobox_interface->insertItem(if_vect[j].c_str());
  ui_combobox_interface->setCurrentItem( 0 );

  if(_pref_if == std::string( tr("manual", "network interface") ))
    {
      ui_lineedit_ip->setText(_last_addr.c_str());
      ui_combobox_interface->setCurrentItem( 1 );
    }
  else
    if( _pref_if == std::string( tr("hostname", "network interface") ) || 
	_pref_if == std::string( tr("file", "network interface") )     ||
	_pref_if == std::string( tr("automatic", "network interface") ) )
      {
	if( _pref_if == std::string( tr("file", "network interface") ) )
	  ui_combobox_interface->setCurrentItem( 2 );
	if( _pref_if == std::string( tr("hostname", "network interface") ) )
	  ui_combobox_interface->setCurrentItem( 3 );
	slotIPUpdate();
      }
    else
      {
	ui_lineedit_ip->setEnabled(false);
	
	std::string if_addr = "127.0.0.1";

	for(j = 0; j < 3; ++j)
	  for(int k = 0; k < if_vect.size(); ++k)
	    if(if_default[j] == if_vect[k])
	      {
		ui_combobox_interface->setCurrentItem(k);
		if_addr = inf.getIfAddress(if_default[j]);
	      }
	
	for(j = 0; j < if_vect.size(); ++j)
	  if(_pref_if == if_vect[j])
	    {
	      ui_combobox_interface->setCurrentItem(j);
	      if_addr = inf.getIfAddress(_pref_if);
	      break;
	    }
	
	if(if_addr.empty()) if_addr = "127.0.0.1";
	ui_lineedit_ip->setText(if_addr.c_str());
      }

  // catch signal 13, PIPE BROKEN
#if !defined(_WIN32)
  struct sigaction sa;
  memset((void *)&sa, 0, sizeof(struct sigaction));
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = 0;
  sa.sa_handler = SIG_IGN;
  sigaction(SIGPIPE, &sa, NULL);
#endif

  // initialize list of shared files
  slotRefreshSharedList();


  search_tabs = 0;
  
  ui_tabwidget_search = new IconTabWidget(page_search, "ui_tabwidget_search");
  connect(ui_tabwidget_search, SIGNAL(iconPressed()), SLOT(slotCloseSearch()));
  connect(ui_tabwidget_search, SIGNAL(sigCurrentChanged(QWidget*)), SLOT(slotCurrentChanged(QWidget*)));
  page_searchLayout->addWidget( ui_tabwidget_search, 1, 0 );

  ui_tabwidget_search->removePage(tab);
  delete tab;
  tab = new SearchWidget(this);
  
  connect(tab, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotSearchDoubleClicked(QListViewItem*)));
  connect(tab, SIGNAL(rightButton(QListViewItem*, const QPoint&, int)), this, SLOT(slotShowSearchContext(QListViewItem*, const QPoint&, int)));
  
  ui_tabwidget_search->addTab(tab, "");
  ui_tabwidget_search->showPage(tab);

  // init IncomingConnect
  _current_port = 6346;
  std::vector<std::string>    str_ports;
  std::vector<unsigned int>   ports;
  StringManipulation          sm(std::string(ui_lineedit_listenport->text()));
  int                         n = 0;

  sm.split_to_vector(str_ports, ',');
  _incoming_connect = NULL;

  do
    {
      std::strstream str_p;
      str_p << str_ports[n++] << std::ends;
      unsigned p;
      str_p >> p;

      _incoming_connect = new IncomingConnect(p, 10);

      if( ! _incoming_connect->ok() ) 
	{
	  delete _incoming_connect;
	  _incoming_connect = NULL;
	}
      else
	{
	  _current_port = p;
	  break;
	}
    }
  while(n < str_ports.size());

  if( ! _incoming_connect )
    {
      std::strstream str;
      str << tr("Cannot bind to ports:") << std::endl << std::endl;
      for(i = 0; i < str_ports.size(); ++i) 
	{
	  str << str_ports[i];
	  if( i+1 < str_ports.size() ) str << std::endl;
	}
      str << std::ends;
      QMessageBox::warning(this, tr("Incomming connection bind error"), str.str(), QMessageBox::Ok, 0);
      str.freeze(false);
    }

  // init hostcache
  if( ui_checkbox_enablehostcache->isChecked() )
    {
      _hostcache = new HostCache(ui_spinbox_hostcacheport->value(), 100, this);
      if( !_hostcache->ok() ) 
	{
	  std::strstream str;
	  str << tr("Cannot bind to port") << " " << ui_spinbox_hostcacheport->value() << std::ends;
	  QMessageBox::warning(this, tr("HostCache bind error"), str.str(), QMessageBox::Ok, 0);
	  str.freeze(false);
	}
    }

  // init upload manager
  upload_manager = new UploadManager(this);

  // init download manager
  download_manager = new DownloadManager(this);

  QObject *o = (QObject*)download_manager;

  connect((QObject*)ui_pushbutton_abortdownload, SIGNAL(clicked()), o, SLOT(slotAbortDownload()));
  connect((QObject*)ui_pushbutton_removedownload, SIGNAL(clicked()), o, SLOT(slotRemoveDownload()));
  connect((QObject*)ui_pushbutton_resumedownload, SIGNAL(clicked()), o, SLOT(slotResumeDownload()));
  connect((QObject*)ui_pushbutton_clearinactive, SIGNAL(clicked()), o, SLOT(slotClearInactive()));

  // read host list
  readHostList();

  // connect if "on start auto-connect" is checked
  if(ui_checkbox_conf_autoconnect->isChecked()) slotConnect();

  // download context menu
  _context_download = new QPopupMenu(this, tr("Download") );
  _context_download->insertItem( tr("Properties"), o, SLOT(slotDownloadProperties()), CTRL+Key_P);
  _context_download->insertItem( tr("Launch"), this, SLOT(slotLaunchSelectedDownloadItem()), CTRL+Key_L);
  _context_download->insertItem( tr("Move into Directory"), o, SLOT(slotMoveDownloads()), CTRL+Key_M);
  _context_download->insertSeparator();
  _context_download->insertItem( tr("Abort"), o, SLOT(slotAbortDownload()), CTRL+Key_A);
  _context_download->insertItem( tr("Remove from list"), o, SLOT(slotRemoveDownload()), CTRL+Key_R);
  _context_download->insertItem( tr("Delete from disc"), this, SLOT(slotDeleteDownload()), CTRL+Key_Delete);
  _context_download->insertItem( tr("Resume"), o, SLOT(slotResumeDownload()), CTRL+Key_S);
  _context_download->insertSeparator();
  _context_download->insertItem( tr("Clear Inactive"), o, SLOT(slotClearInactive()), CTRL+Key_C);

  // begin: the following slots are implemented in QtellaSubSlotsContext.cpp
  // search context menu
  _context_search = new QPopupMenu(this, tr("Search") );
  _context_search->insertItem( tr("Download"), this, SLOT(slotDownloadAll()), CTRL+Key_D, 0);
  _context_search->insertItem( tr("Download to"), this, SLOT(slotDownloadTo()), CTRL+Key_T, 4);
  _context_search->insertSeparator();
  _context_search->insertItem( tr("Stop Download"), this, SLOT(slotSearchStopDownload()), CTRL+Key_A, 1);
  _context_search->insertItem( tr("Remove from Download List"), this, SLOT(slotSearchRemoveDownload()), CTRL+Key_R, 2 );
  _context_search->insertItem( tr("Delete from disc"), this, SLOT(slotSearchDeleteDownload()), CTRL+Key_Delete, 3 );

  // upload context menu
  _context_upload = new QPopupMenu(this, tr("Upload") );
  _context_upload->insertItem( tr("Abort"), this, SLOT(slotAbortUpload()), CTRL+Key_A);
  _context_upload->insertItem( tr("Remove"), this, SLOT(slotRemoveUpload()), CTRL+Key_Delete);
  _context_upload->insertItem( tr("Block host"), this, SLOT(slotUploadBlockHost()), CTRL+Key_B);
  _context_upload->insertSeparator();
  _context_upload->insertItem( tr("Clear Inactive"), this, SLOT(slotClearInactiveUploads()), CTRL+Key_I);

  // shared files context menu
  _context_shared = new QPopupMenu(this, tr("Shared Files") );
  _context_shared->insertItem( tr("Launch"), this, SLOT(slotSharedFilesLaunch()), CTRL+Key_L);
  _context_shared->insertItem( tr("Properties"), this, SLOT(slotSharedProperties()), CTRL+Key_P);
  _context_shared->insertItem( tr("Update List"), this, SLOT(slotRefreshSharedList()), CTRL+Key_U);
  _context_shared->insertItem( tr("Delete from disc"), this, SLOT(slotDeleteSharedFile()), CTRL+Key_Delete);
  _context_shared->insertItem( tr("Rename file"), this, SLOT(slotRenameSharedFile()), CTRL+Key_N);
  // end

  // connect context menu
  _context_connect = new QPopupMenu(this, tr("Connections") );
  _context_connect->insertItem( tr("Add to AutoConnect"), this, SLOT(slotAddHostAutoConnect()), CTRL+Key_A );
  _context_connect->insertItem( tr("Disconnect"), (QObject*)connections, SLOT(slotRemoveHost()), CTRL+Key_D );
  _context_connect->insertItem( tr("Block Host"), (QObject*)connections, SLOT(slotBlockHost()), CTRL+Key_B );
  _context_connect->insertItem( tr("Show Handshake"), (QObject*)connections, SLOT(slotShowHandshake()), CTRL+Key_S );

  _interrupted_downloads = new InterruptedDownloads(this, std::string(ui_lineedit_incompleted->text()));
  connect(ui_button_refreshint, SIGNAL(pressed()), _interrupted_downloads, SLOT(update()));

  ui_pushbutton_properties->hide();

#ifndef KDE
  ui_checkbox_fileassociations->hide();
  ui_checkbox_systemtray->hide();
#endif

  // version check

  unsigned long vers = 0;
  if( _prev_version.empty() ) vers = 0;
  else
    {
      std::vector<std::string>     v;
      StringManipulation           sm(_prev_version);
      std::string                  version;

      sm.split_to_vector(v, '.');
      for(i = 0; i < v.size(); ++i)
	{
	  std::string::size_type idx = v[i].find_first_not_of("0123456789");
	  if(idx != std::string::npos) v[i] = v[i].substr(0, idx);

	  for(j = 0; j < (3 - v[i].size()); ++j) version += std::string("0");
	  version += v[i];
	}
      
      std::strstream str;
      str << version << std::ends;
      str >> vers;
    }

  if( vers < 3005 ) 
    {
      std::strstream str;
      str << "Since version 0.3.5 a new filename format for" << std::endl
	  << "interrupted downloads has been introduced. Before" << std::endl
	  << "starting any downloads you should convert the old" << std::endl
	  << "format to the new one." << std::endl
	  << "Select \"Convert Filenames\" from the \"File\" menu." << std::ends;

      QMessageBox::information(this, "New Filename Format", str.str(), QMessageBox::Ok, 0);
    }
}


const int QtellaSub::strtoint(const std::string& s)
{
  int            i;
  std::strstream str;
  str << s << std::ends;
  str >> i;
  return i;
}


void QtellaSub::readHostList()
{
  if(!ui_checkbox_savehostlist->isChecked()) return;

  QFile f(ui_lineedit_hostlistfile->text());

  if(f.open(IO_ReadOnly))
    {
      char buffer[64];
      int  r;
      while( (r = f.readLine(buffer, 64)) > 0)
	{
	  std::strstream str;
	  str << buffer << std::ends;
	  std::string s;
	  str >> s;
	  ui_listbox_hostlist->insertItem(s.c_str());
	}
      f.close();
    }
  else
    std::cerr << "Cannot open host list file" << std::endl;
}


QtellaSub::~QtellaSub()
{
  delete _incoming_connect;
  delete upload_manager;
  delete download_manager;
  delete _hostcache;

  // remove all files on exit ?
  if(ui_checkbox_conf_removeall->isChecked()) removeFiles(-1);

  // remove files with size less/equal than x ?
  if(ui_checkbox_conf_removefiles->isChecked()) removeFiles(ui_spinbox_conf_filesize->value());

  // save configuration
  slotApplyConfig();

  // save host list
  slotDisconnect();

  if(ui_checkbox_savehostlist->isChecked())
    {
      QFile f(ui_lineedit_hostlistfile->text());
      
      if(f.open(IO_WriteOnly))
	{
	  for(int i = 0; i < ui_listbox_hostlist->count(); ++i)
	    {
	      std::string s = static_cast<std::string>(ui_listbox_hostlist->text(i));
	      s += "\n";
	      f.writeBlock(s.data(), s.size());
	    }
	  f.close();
	}
      else
	std::cerr << "Error opening host list file." << std::endl;
    }
}


void QtellaSub::slotAddHost()
{
  if(!timer_connection->isActive())
    timer_connection->start(500);
  connections->slotAddHost();
}


void QtellaSub::check_connections()
{
  connections->checkServants();
}

void QtellaSub::slotAddDirectory()
{
  QString dirName =
    QFileDialog::getExistingDirectory(0, this, 0, tr("Select shared directory"), true);

  if ( !dirName.isEmpty() ) ui_listbox_shareddirectories->insertItem( dirName );

  // check for already being shared directories
  std::vector<std::string> dirs;
  std::vector<std::string> removed;
  int i;

  for(i = 0; i < ui_listbox_shareddirectories->count(); ++i)
    {
      bool append = true;
      for(int j = 0; j < dirs.size(); ++j) 
	{
	  std::string s( ui_listbox_shareddirectories->text(i) );
	  if( (s.find(dirs[j]) == 0) || (dirs[j].find(s) == 0) )
	    {
	      append = false;
	      if( dirs[j].size() > s.size() ) 
		{
		  removed.push_back(dirs[j]);
		  dirs[j] = s;
		}
	      else removed.push_back(s);
	    }
	}
      if(append) dirs.push_back( std::string(ui_listbox_shareddirectories->text(i)) );
    }

  ui_listbox_shareddirectories->clear();
  
  for(i = 0; i < dirs.size(); ++i) ui_listbox_shareddirectories->insertItem( dirs[i].c_str() );

  if( ! removed.empty() )
    {
      std::strstream str;
      str << tr("These directories are already being shared:") << "\n\n";
      for(i = 0; i < removed.size(); ++i)
	str << removed[i] << "\n";
      str << std::ends;
      QMessageBox::warning(this, tr("Removed Directories"), str.str(), QMessageBox::Ok, 0);
      str.freeze(false);
    }

  slotRefreshSharedList();
}

void QtellaSub::slotRemoveDirectory()
{
  int index = ui_listbox_shareddirectories->currentItem();

  if(index>-1)
    ui_listbox_shareddirectories->removeItem(index);

  slotRefreshSharedList();
}

void QtellaSub::checkDirectories(const std::string& directory, std::vector<SharedFile*> *old)
{
  QDir   dir(directory.c_str());

  if(dir.exists())
    {
      QStringList dirlist = dir.entryList(QDir::Dirs);
      QStringList::Iterator it;

      for( it = dirlist.begin(); it != dirlist.end(); ++it )
	{
	  std::string filename = (*it).latin1();
	  if(filename!="." && filename!="..")
	    checkDirectories( (directory+"/"+filename), old );
	}

      QStringList list = dir.entryList(QDir::Files);
      
      for( it = list.begin(); it!=list.end(); ++it )
	{
	  std::string filename = (*it).latin1();
	  
	  std::string complete = directory;
	  if( complete[complete.size()-1] != '/' ) complete += "/";
	  complete += filename;
	  
	  QFile f(complete.c_str());
	  
	  if(f.exists())
	    {
	      //insert filename into the hash table
	      _hash_table.insert(filename);
	      //insert filesize into set of filesizes
	      _sharedsize_set.insert(f.size());

	      SharedViewItem *item = new SharedViewItem(ui_listview_shared, filename.c_str());
		  
	      item->setText(1, "0");  // uploads
	      item->setText(2, "0");  // requests
	      
	      std::strstream stream_fsize;
	      stream_fsize << f.size() << std::ends;
	      std::string    str(stream_fsize.str());
	      item->setText(3, StringManipulation::insertDelimiter(str, ',', 3).c_str() );
	      stream_fsize.freeze(false);
		  
	      vSharedFiles.push_back(new SharedFile(item, filename, directory, f.size()));
		  
	      for(int i = 0; i < (*old).size(); i++)
		if( *((*old)[i]) == *vSharedFiles.back() )
		  {
		    int up = (*old)[i]->uploads;
		    int rq = (*old)[i]->requests;
		    
		    vSharedFiles.back()->uploads = up;
		    vSharedFiles.back()->requests = rq;
		    
		    std::strstream str;
		    str << up << std::ends;
		    item->setText(1, str.str());
		    str.freeze(false);
		    str.seekp(0, std::ios::beg);
		    str << rq << std::ends;
		    item->setText(2, str.str());
		    str.freeze(false);
		  }
	    }
	}
    }
}


void QtellaSub::downloadItem(QListViewItem *item, const std::string& directory)
{
  for(int m = 0; m < connections->_search.size(); m++)
    for(int i = 0; i < connections->_search[m]->vSearchResults.size(); i++)
      if( connections->_search[m]->vSearchResults[i]->item == item) 
	{
	  QueryHitEntry q = *(connections->_search[m]->vSearchResults[i]);

	  // item must be set to NULL to prevent the deletion of the item
	  // by the constructor of the Download
	  q.item = NULL;

	  download_manager->addDownload(q, item, "", NULL, directory);
	}
}


void QtellaSub::slotSearchDoubleClicked(QListViewItem *item)
{
  downloadItem(item);
}


void QtellaSub::slotSearch()
{
  std::string query = static_cast<std::string>(ui_combobox_search->currentText());
  search(query);
}


const void QtellaSub::search(const std::string& query)
{
  if(query.size() < 2) return;

  bool found = false;
  for(int i = 0; i < ui_combobox_search->count(); ++i)
    if( ui_combobox_search->text(i) == ui_combobox_search->currentText() ) found = true;
  if(!found) ui_combobox_search->insertItem(ui_combobox_search->currentText(), 0);

  QPixmap remove_research(*(Icon::getInstance()->getRemoveResearch()));

  SearchWidget *c = (SearchWidget*) ui_tabwidget_search->currentPage();
  if( c ) _search_column_size = c->ui_listview_search->columnWidth(2);

  if(search_tabs == 0)
    {
      ui_tabwidget_search->changeTab(ui_tabwidget_search->currentPage(), QIconSet(remove_research), query.c_str());
      connections->search(query, (SearchWidget*)(ui_tabwidget_search->currentPage()) );
    }
  else
    {
      SearchWidget *s = new SearchWidget(this);

      connect(s, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotSearchDoubleClicked(QListViewItem*)));
      connect(s, SIGNAL(rightButton(QListViewItem*, const QPoint&, int)), this, SLOT(slotShowSearchContext(QListViewItem*, const QPoint&, int)));
      ui_tabwidget_search->addTab(s, QIconSet(remove_research), query.c_str());
      ui_tabwidget_search->showPage(s);

      connections->search(query, s);
    }

  search_tabs++;
  ui_pushbutton_stopsearch->setEnabled(true);
  ui_pushbutton_stopsearch->setText( tr("Stop", "button text to stop the search") );
}


void QtellaSub::slotAddAutoConnect()
{
  std::string s = (const char*)ui_lineedit_autoconnect->text();
  
  if(!s.empty())
    ui_listbox_autoconnect->insertItem(s.c_str());

  slotApplyConfig();
  ui_lineedit_autoconnect->setText("");
}


void QtellaSub::slotRemoveAutoConnect()
{
  int index = ui_listbox_autoconnect->currentItem();

  if(index>-1)
    ui_listbox_autoconnect->removeItem(index);

  slotApplyConfig();
}


void QtellaSub::sighandler(int i)
{
  #ifdef QTELLA_WARNINGS
    std::cout << "signal: " << i << std::endl;
  #endif
}


void QtellaSub::slotReadFile()
{
  QFileDialog d(this, 0, true);

  if(d.exec())
    {
      std::string s = static_cast<const char*>(d.selectedFile());

      std::fstream f;

      f.open(s.c_str(), std::ios::in );
      if(f.is_open())
	{
	  while( !f.eof() )
	    {
	      char   buffer[1024];
	      f.getline(buffer, 1023);
	      std::string input = buffer;
	      
	      if(input.find(":") != std::string::npos)
		ui_listbox_hostlist->insertItem( input.c_str(), 0);
	    }
	}
    }
}



void QtellaSub::slotStopSearch()
{
  SearchWidget* sw  = (SearchWidget*) ui_tabwidget_search->currentPage();

  std::vector<Search*>::iterator pos = connections->_search.begin();
  while((*pos)->sw != sw) ++pos;

  if(std::string(ui_pushbutton_stopsearch->text()) == std::string( tr("Stop", "button text to stop the search") ) )
    {
      (*pos)->active = false;
      ui_pushbutton_stopsearch->setText( tr("Delete", "button text to delete search tab") );
    }
  else
    {
      for(int j = 0; j < (*pos)->vSearchResults.size(); j++)
	{
	  _interrupted_downloads->removeQueryHit((*pos)->vSearchResults[j]->size);
	  delete (*pos)->vSearchResults[j];
	}

      connections->_search.erase(pos);

      if(search_tabs==1)
	{
	  sw->ui_listview_search->clear();
	  ui_tabwidget_search->changeTab(sw, QIconSet(0), "");
	  ui_pushbutton_stopsearch->setText( tr("Stop", "button text to stop the search") );
	  ui_pushbutton_stopsearch->setEnabled(false);
	}
      else
	{
	  ui_tabwidget_search->removePage(sw);
	  delete sw;

	  sw = (SearchWidget*) ui_tabwidget_search->currentPage();
	  pos = connections->_search.begin();
	  while((*pos)->sw != sw) ++pos;
	  if((*pos)->active)
	    ui_pushbutton_stopsearch->setText( tr("Stop", "button text to stop the search") );
	  else
	    ui_pushbutton_stopsearch->setText( tr("Delete", "button text to delete search tab") );
	}

      search_tabs--;
    }
}


void QtellaSub::slotCurrentChanged(QWidget* w)
{
  std::vector<Search*>::iterator pos = connections->_search.begin();
  while( pos != connections->_search.end() ) 
    {
      if( (*pos)->sw == (SearchWidget*)w ) break;
      ++pos;
    }

  if(pos != connections->_search.end())
    if((*pos)->active)
      ui_pushbutton_stopsearch->setText( tr("Stop", "button text to stop the search") );
    else
      ui_pushbutton_stopsearch->setText( tr("Delete", "button text to delete search tab") );
}


void QtellaSub::removeFiles(const int size)
{
  long              lsize = size;
  std::string       directory(ui_lineedit_incompleted->text());

  if(size>-1) lsize = size << 10;
  QDir   dir(directory.c_str());

  if(directory.size()>0)
    if(directory[directory.size()-1]!='/') directory += "/";

  if(dir.exists())
    {
      QStringList list = dir.entryList(QDir::Files);
      
      for( QStringList::Iterator it = list.begin(); it!=list.end(); ++it )
	{
	  std::string filename = (*it).latin1();
	  std::string complete(directory+filename);
	  
	  QFile f(complete.c_str());

	  if(filename.find(".part_") != std::string::npos)
	    if(f.exists())
	      if( (f.size() <= lsize) || (lsize==-1) ) f.remove();
	}
    }
}


void QtellaSub::launchFile(const std::string& filename)
{
  
#ifdef KDE

  if( ! ui_checkbox_fileassociations->isChecked() )
    {
      KFileOpenWithHandler open_with;
      new KRun(KURL(filename.c_str()));
      return;
    }
  else
    {

#endif
  
      std::string::size_type idx = filename.find_last_of(".");
      std::string            ext(filename.substr(idx+1));
      std::string            application;
      
      for(int i = 0; i < _applications.size(); ++i)
	if(_applications[i].first.find(ext) != std::string::npos) application = _applications[i].second;

      if(application.empty()) return;
      
#if !defined(_WIN32)
      if(!fork())
	{
	  execlp( application.c_str(), _binary.c_str(), filename.c_str(), NULL );
	  exit(0);
	}
#endif
  
#ifdef KDE
    }
#endif
  
}


void QtellaSub::slotSharedFilesLaunch(QListViewItem* item)
{
  std::vector<SharedFile*>::iterator    pos = vSharedFiles.begin();

  while(pos != vSharedFiles.end()) if(item == (*pos)->item) break; else ++pos;
  launchFile( (*pos)->directory + "/" + (*pos)->file );
}


void QtellaSub::launchFileList(std::vector<std::string>& files)
{
  int i;
  if(files.empty()) return;

#ifdef KDE

  if( ! ui_checkbox_fileassociations->isChecked() )
    {
      KFileOpenWithHandler open_with;
      for(i = 1; i <= files.size(); ++i) new KRun(KURL(files[i-1].c_str()));
      return;
    }
  else
    {

#endif
  
      std::string::size_type idx = files[0].find_last_of(".");
      std::string ext(files[0].substr(idx+1));
      std::string application;
      
      for(i = 0; i < _applications.size(); ++i)
	if(_applications[i].first.find(ext) != std::string::npos) application = _applications[i].second;
      
      if(application.empty()) return;
      
#if !defined(_WIN32)
      if(!fork())
	{
	  const int N = 1000;
	  char*     args[N];
	  
	  for(i = 1; i <= files.size(); ++i) 
	    if(i < N-1) args[i] = const_cast<char*>(files[i-1].c_str());
	  
	  args[0] = const_cast<char*>(_binary.c_str());
	  if(files.size()+1 < N) args[files.size()+1] = NULL; else args[N-1] = NULL;
	  
	  execvp( application.c_str(), args );
	  exit(0);
	}
#endif

#ifdef KDE
    }  
#endif

}


void QtellaSub::slotAddApplication()
{
  AddApplication aa(this, "", true);

  if(aa.exec())
    {
      new QListViewItem(ui_listview_apps, aa.ui_lineedit_type->text(), aa.ui_lineedit_application->text());
      _applications.push_back( std::make_pair(aa.ui_lineedit_type->text(), aa.ui_lineedit_application->text()) );
    }
}


void QtellaSub::slotEditApplication()
{
  QListViewItem*    fc = ui_listview_apps->firstChild();
  
  while(fc)
    if(fc->isSelected())
      {
	AddApplication aa(this, "", true);
	aa.ui_lineedit_type->setText(fc->text(0));
	aa.ui_lineedit_application->setText(fc->text(1));
	if(aa.exec())
	  {
	    int i;
	    for(i = 0; i < _applications.size(); ++i)
	      if(_applications[i].first == std::string(fc->text(0))) break;
	    
	    fc->setText(0, aa.ui_lineedit_type->text());
	    fc->setText(1, aa.ui_lineedit_application->text());
	    _applications[i] = std::make_pair(aa.ui_lineedit_type->text(), aa.ui_lineedit_application->text());
	  }
	break;
      }
    else
      fc = fc->itemBelow();
}


void QtellaSub::slotRemoveApplication()
{
  QListViewItem*    fc = ui_listview_apps->firstChild();
  QListViewItem*    tmp;

  while(fc)
    if(fc->isSelected())
      {
	tmp = fc;
	std::string type(fc->text(0));
	std::string app(fc->text(1));
	std::vector< std::pair<std::string, std::string> >::iterator     pos = _applications.begin();
	while( ((*pos).first != type) && ((*pos).second != app) ) ++pos;
	_applications.erase(pos);
	fc = tmp->itemBelow();
	delete tmp;
      }
    else
      fc = fc->itemBelow();
}


void QtellaSub::slotDownloadDoubleClicked(QListViewItem* item)
{
  if(!item) return;

  int i = 0;

  for(i = 0; i < download_manager->_downloads.size(); ++i)
    if(item == (QListViewItem*)download_manager->_downloads[i]->_item) 
      {
	if(download_manager->_downloads[i]->_f) 
	  launchFile( std::string(download_manager->_downloads[i]->_f->name()) );
	break;
      }
}


void QtellaSub::slotLaunchSelectedDownloadItem()
{
  for(int i = 0; i < download_manager->_downloads.size(); ++i)
    if(download_manager->_downloads[i]->_item->isSelected())
      {
	launchFile( std::string(download_manager->_downloads[i]->_f->name()) );
	break;
      }
}


void QtellaSub::slotDownloadShowContext(QListViewItem* item, const QPoint& p, int column)
{
  _context_download->popup(p);
}


void QtellaSub::slotShowSearchContext(QListViewItem* item, const QPoint& p, int column)
{
  _context_search->setItemEnabled(0, true);    // download
  _context_search->setItemEnabled(1, false);   // stop
  _context_search->setItemEnabled(2, false);   // remove
  _context_search->setItemEnabled(3, false);   // delete

  SearchWidget *s = (SearchWidget*) ui_tabwidget_search->currentPage();

  for(int j = 0; j < connections->_search.size(); j++) 
    if(connections->_search[j]->sw == s)
      {
	for(int i = 0; i < connections->_search[j]->vSearchResults.size(); i++)
	  if( connections->_search[j]->vSearchResults[i]->item->isSelected() )
	    {
	      if( download_manager->exists(*connections->_search[j]->vSearchResults[i]) )
		{
		  _context_search->setItemEnabled(0, false);  // download
		  _context_search->setItemEnabled(1, true);   // stop
		  _context_search->setItemEnabled(2, true);   // remove
		  _context_search->setItemEnabled(3, true);   // delete
		}
	      break;
	    }
	
	break;
      }
  
  _context_search->popup(p);
}


void QtellaSub::slotShowUploadContext(QListViewItem* item, const QPoint& p, int column)
{
  _context_upload->popup(p);
}


void QtellaSub::slotShowSharedContext(QListViewItem* item, const QPoint& p, int column)
{
  _context_shared->popup(p);
}


void QtellaSub::slotHostCacheChangePort(int port)
{
  if(_hostcache) 
    {
      delete _hostcache;
      _hostcache = new HostCache(port, 100, this);
    }
}

void QtellaSub::slotHostCacheChangeEnable()
{
  if( _read_config ) return;

  if(_hostcache)
    {
      delete _hostcache;
      _hostcache = NULL;
    }
  else
    _hostcache = new HostCache(ui_spinbox_hostcacheport->value(), 100, this);
}

QtellaSub* QtellaSub::getInstance()
{
  return _instance;
}

void QtellaSub::slotClickedAutoConnect(int i, QListBoxItem* item, const QPoint& p)
{
  if(item) ui_lineedit_autoconnect->setText(item->text());
}

void QtellaSub::slotConnectContext(QListViewItem* item, const QPoint& p, int i)
{
  _context_connect->popup(p);
}

void QtellaSub::slotAddHostAutoConnect()
{
  QListViewItem* item = ui_listview_connections->selectedItem();
  if(item)
    ui_listbox_autoconnect->insertItem(item->text(0));
}

void QtellaSub::slotUpdateAutoConnect()
{
  int i = ui_listbox_autoconnect->currentItem();
  if(i != -1)
    ui_listbox_autoconnect->changeItem(ui_lineedit_autoconnect->text(), i);
}

void QtellaSub::slotChangeFinished()
{
  QString s = QFileDialog::getExistingDirectory(NULL, this, "", tr("Directory for finished downloads"), true);
  if(s != QString::null) ui_lineedit_downloaddirectory->setText(s);
}

void QtellaSub::slotChangeInterrupted()
{
  QString s = QFileDialog::getExistingDirectory(NULL, this, "", tr("Directory for incomplete downloads"), true);
  if(s != QString::null) ui_lineedit_incompleted->setText(s);
}

void QtellaSub::slotChangeHostlist()
{
  QFileDialog d(this, 0, true);
  if(d.exec()) ui_lineedit_hostlistfile->setText(d.selectedFile());
}

void QtellaSub::slotChangeLogfile()
{
  QFileDialog d(this, 0, true);
  if(d.exec()) ui_lineedit_logfile->setText(d.selectedFile());
}

void QtellaSub::slotSelectHostCacheLogFile()
{
  QFileDialog d(this, 0, true);
  if(d.exec()) ui_lineedit_hostcachelogfile->setText(d.selectedFile());
}

void QtellaSub::slotDeleteDownload()
{
  if( ! _show_delete_dialog )
    {
      download_manager->slotDelete();
      return;
    }

  DeleteDialog dd(this, 0, true);

  if( dd.exec() ) download_manager->slotDelete();
  _show_delete_dialog = ! dd.ui_checkbox_askagain->isChecked();
}

void QtellaSub::slotCloseSearch()
{
  SearchWidget * sw  = (SearchWidget*) ui_tabwidget_search->currentPage();

  if( ui_checkbox_group_size->isChecked() )
    slotGroupSize(0);

  std::vector<Search*>::iterator pos = connections->_search.begin();

  while((*pos)->sw != sw) ++pos;

  (*pos)->active = false;
  
  for(int j = 0; j < (*pos)->vSearchResults.size(); j++) 
    {
      _interrupted_downloads->removeQueryHit((*pos)->vSearchResults[j]->size);
      delete (*pos)->vSearchResults[j];
    }

  connections->_search.erase(pos);

  if(search_tabs==1)
    {
      sw->ui_listview_search->clear();
      ui_tabwidget_search->changeTab(sw, QIconSet(0), "");
      ui_pushbutton_stopsearch->setEnabled(false);
    }
  else
    {
      ui_tabwidget_search->removePage(sw);
      delete sw;
      
      sw = (SearchWidget*) ui_tabwidget_search->currentPage();
      pos = connections->_search.begin();
    }
  search_tabs--;
}


void QtellaSub::slotChangeInterface(const QString& s)
{
  slotIPUpdate();
//   _pref_if = std::string(s);

//   if(_pref_if == "manual") 
//     ui_lineedit_ip->setEnabled(true);
//   else 
//     ui_lineedit_ip->setEnabled(false);

//   Interface i;
//   std::string t = i.getIfAddress(_pref_if);
//   if(t.empty()) t = "127.0.0.1";
//   ui_lineedit_ip->setText(t.c_str());
}


void QtellaSub::slotLaunchInterruptedFile()
{
  std::vector<InterruptedFile> files;
  _interrupted_downloads->getSelectedFiles(files);

  std::vector<std::string> f;
  for(int i = 0; i < files.size(); ++i) f.push_back(files[i].completeName());
  launchFileList(f);
}


void QtellaSub::slotDeleteInterruptedFile()
{
  _interrupted_downloads->deleteSelectedFiles();
}


void QtellaSub::slotSearchInterruptedFile()
{
  _interrupted_downloads->searchSelectedFiles();
}


void QtellaSub::slotResumeInterruptedFile()
{
  _interrupted_downloads->resumeSelectedFiles();
}


void QtellaSub::slotAbortInterruptedFile()
{
  _interrupted_downloads->abortSelectedFiles();
}

void QtellaSub::slotInterruptedFileContext(QListViewItem* i, const QPoint& p, int j)
{
  _interrupted_downloads->slotShowContextMenu((InterruptedViewItem*)i, p, j);
}


// --

void QtellaSub::slotResubmitSearch()
{
  if( connections->_search.empty() ) return;

  if( ui_checkbox_group_size->isChecked() )
    slotGroupSize(0);

  SearchWidget* sw  = (SearchWidget*) ui_tabwidget_search->currentPage();

  std::vector<Search*>::iterator pos = connections->_search.begin();
  while((*pos)->sw != sw) ++pos;
  std::string query = (*pos)->query;

  ui_pushbutton_stopsearch->setText( tr("Stop", "button text to stop the search") );
  QPixmap remove_research(*(Icon::getInstance()->getRemoveResearch()));
  ui_tabwidget_search->changeTab(sw, QIconSet(remove_research), query.c_str());

  for(int j = 0; j < (*pos)->vSearchResults.size(); j++)
    {
      _interrupted_downloads->removeQueryHit((*pos)->vSearchResults[j]->size);
      delete (*pos)->vSearchResults[j];
    }

  connections->_search.erase(pos);
  connections->search( query, sw );
}


void QtellaSub::slotIPUpdate()
{
  std::string iface(ui_combobox_interface->currentText());

  _ip_update_timer->changeInterval(ui_spinbox_updateinterval->value() * 60000);

  if( iface == std::string( tr("manual", "network interface") ) )
    {
      ui_pushbutton_updateip->setEnabled(false);
      ui_lineedit_hostname->setEnabled(false);
      ui_lineedit_ip->setEnabled(true);
      ui_spinbox_updateinterval->setEnabled(false);
      return;
    }

  if( iface == std::string( tr("hostname", "network interface") ) )
    {
      ui_pushbutton_updateip->setEnabled(true);
      ui_lineedit_hostname->setEnabled(true);
      ui_lineedit_ip->setEnabled(false);
      ui_spinbox_updateinterval->setEnabled(true);

      ui_lineedit_ip->setText( Address::getIPOfHost((const char*)ui_lineedit_hostname->text()).c_str() );
      return;
    }

  if( iface == std::string( tr("file", "network interface") ) )
    {
      ui_pushbutton_updateip->setEnabled(true);
      ui_lineedit_hostname->setEnabled(true);
      ui_lineedit_ip->setEnabled(false);
      ui_spinbox_updateinterval->setEnabled(true);

      std::fstream   f;
      f.open( (const char*)(ui_lineedit_hostname->text()), std::ios::in );
      if( f.is_open() )
	{
	  std::string ip;
	  f >> ip;
	  ui_lineedit_ip->setText( ip.c_str() );
	}

      return;
    }

  if( iface == std::string( tr("automatic", "network interface") ) )
    {
#ifdef _DEBUG
      qDebug( "QtellaSub:: automatic selected" );
#endif
      ui_lineedit_hostname->setEnabled(false);
      ui_lineedit_ip->setEnabled(false);
      ui_pushbutton_updateip->setEnabled(false);
      ui_spinbox_updateinterval->setEnabled(false);

      ui_lineedit_ip->setText( _last_addr.c_str() );

      return;
    }


  ui_pushbutton_updateip->setEnabled(true);
  ui_lineedit_hostname->setEnabled(false);
  ui_spinbox_updateinterval->setEnabled(true);

#if defined(_WIN32)
  // for windows the selected interface is the ip address
  ui_lineedit_ip->setText(iface.c_str());
#else
  // for linux/unix the selected interface is a network interface
  Interface i;
  ui_lineedit_ip->setText(i.getIfAddress(iface).c_str());
#endif
}


void QtellaSub::slotAddBlockedHost()
{
  BlockHost bh(0, 0, true);
  
  if( bh.exec() == QDialog::Accepted )
    ui_listbox_blockedhosts->insertItem(bh.ui_lineedit_ip->text());
}


void QtellaSub::slotDeleteBlockedHost()
{
  int i = 0;

  while( i < ui_listbox_blockedhosts->count() )
    if( ui_listbox_blockedhosts->isSelected(i) )
      ui_listbox_blockedhosts->removeItem(i);
    else
      ++i;
}


// system tray patch
void QtellaSub::closeEvent(QCloseEvent *e)
{
#ifdef KDE
  if( ui_checkbox_systemtray->isChecked() )
    if( isVisible() )
      hide();
    else 
      QWidget::closeEvent(e);
  else
    QWidget::closeEvent(e);
#else
  QWidget::closeEvent(e);
#endif
}


void QtellaSub::slotGroupSize(int state)
{
  // 2 = on
  // 0 = off

  //   while((*pos)->sw != sw) ++pos;
  //   SearchWidget* sw  = (SearchWidget*) ui_tabwidget_search->currentPage();

  std::vector<Search*>::iterator pos = connections->_search.begin();

  for( pos; pos != connections->_search.end(); ++pos )
    {
      if( state == 0 )
	{
	  QListViewItem* item = (*pos)->sw->ui_listview_search->firstChild();

	  while( item )
	    {
	      QListViewItem* child = item->firstChild();
	      while( child )
		{
		  child->moveItem( item );
		  child = item->firstChild();
		}
	      item = item->nextSibling();
	    }
	  
	}
      else
	if ( state == 2 )
	  {
	    Search::SIZEMAP m;
	    Search::SIZEMAP::iterator p = (*pos)->_size_map.begin();
	    
	    while( p != (*pos)->_size_map.end() )
	      {
		static_cast<SearchViewItem*>(p->second->item)->_group_parent = false;

		Search::SIZEMAP::iterator n = m.find( p->first );

		if( n == m.end() )
		  {
		    m.insert( std::make_pair( p->first, p->second ) );
		  }
		else
		  {
		    static_cast<SearchViewItem*>( n->second->item )->_group_parent = true;

		    (*pos)->sw->ui_listview_search->takeItem( p->second->item);
		    n->second->item->insertItem( p->second->item );
		  }

		++p;
	      }
	  }
    }
}
