#include "QtellaSub.h"
#include "StringManipulation.h"
#include "SearchWidget.h"

#include <string>
#include <fstream>
#include <strstream>
#include <iostream>
#include <vector>

#include "qdir.h"
#include "qlineedit.h"
#include "qcheckbox.h"
#include "qlistview.h"
#include "qspinbox.h"
#include "qlistbox.h"
#include "qcombobox.h"
#include "qradiobutton.h"
#include "qtabwidget.h"
#include "qpopupmenu.h"

#undef _DEBUG

void QtellaSub::readConfiguration()
{
  _show_delete_dialog = true;
  _read_config = true;
  _last_addr = "0.0.0.0";

  QDir d;
  std::string homedir = static_cast<std::string>(d.homeDirPath());
      
  ui_lineedit_downloaddirectory->setText(homedir.c_str());
  ui_lineedit_incompleted->setText(homedir.c_str());

#if !defined(_WIN32)
  ui_lineedit_hostlistfile->setText( (homedir + "/hostlist").c_str() );
#else
  ui_lineedit_hostlistfile->setText( "c:\\hostlist" );
#endif

#if !defined(_WIN32)
  conffile = homedir + "/.qtella";
#else
  conffile = "c:\\qtella.ini";
#endif
  std::fstream   f;
  int            i;

  f.open(conffile.c_str(), std::ios::in );
  if(f.is_open())
    {
      while( !f.eof() )
	{
	  char   buffer[1024];
	  f.getline(buffer, 1023);
	  std::string input = buffer;
	  
	  StringManipulation s(input);
	  
	  s.remove_comments();
	  
	  std::string s1, s2;
	  s.split(s1, s2, '=');
	  
	  s.assign(s1);
	  s.remove_chars("\n ");
	  s1 = s.str();
	  
	  s.assign(s2);
	  s.remove_chars("\n");
	  s2 = s.str();
	  
	  //transform(s1.begin(), s1.end(), s1.begin(), toupper);
	  for(i = 0; i < s1.size(); ++i) s1[i] = toupper(s1[i]);

	  if(!s1.empty())
	    {
	      if(s1=="LAST_DL_TO_DIR") _last_download_to_dir = s2;
	      if(s1=="LASTADDR") _last_addr = s2;
	      if(s1=="IFACE") _pref_if = s2;
	      if(s1=="IMAGES") ui_lineedit_images->setText(s2.c_str());
	      if(s1=="AUDIO") ui_lineedit_audio->setText(s2.c_str());
	      if(s1=="VIDEOS") ui_lineedit_video->setText(s2.c_str());
	      if(s1=="LOGFILE") ui_lineedit_logfile->setText(s2.c_str());
	      if(s1=="DOWNLOAD") 
		if( !s2.empty() ) ui_lineedit_downloaddirectory->setText(s2.c_str());
	      if(s1=="INCOMPLETE") 
		if( !s2.empty() ) ui_lineedit_incompleted->setText(s2.c_str());
	      if(s1=="PORT") ui_lineedit_listenport->setText(s2.c_str());
	      if(s1=="HOST_LIST_FILE") 
		if( !s2.empty() ) ui_lineedit_hostlistfile->setText(s2.c_str());
	      if(s1=="HOSTCACHE_LOGFILE") ui_lineedit_hostcachelogfile->setText(s2.c_str());
	      if(s1=="VERSION") _prev_version = s2;
	      if(s1=="HOSTNAME") ui_lineedit_hostname->setText(s2.c_str());
	      if(s1=="STYLE") _style_s = s2;

	      if(s1=="TABSIZE_DOWNLOAD") ui_listview_download->setColumnWidth(0, strtoint(s2));
	      if(s1=="TABSIZE_SHARED") ui_listview_shared->setColumnWidth(0, strtoint(s2));
	      if(s1=="TABSIZE_UPLOADS") ui_listview_uploads->setColumnWidth(0, strtoint(s2));
	      if(s1=="TABSIZE_INTERRUPT") ui_listview_interrupt->setColumnWidth(0, strtoint(s2));
	      if(s1=="TABSIZE_SEARCH") _search_column_size = strtoint(s2);

	      if(s1=="SHAREDDIR")
		{
		  std::vector<std::string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ';');
		  for(i=0; i<vString.size(); i++)
		    ui_listbox_shareddirectories->insertItem(vString[i].c_str());
		}

	      if(s1=="APPS")
		{
		  std::vector<std::string> v;
		  s.assign(s2);
		  s.split_to_vector(v, ':');
		  _applications.push_back( std::make_pair(v[0], v[1]) );
		  new QListViewItem(ui_listview_apps, v[0].c_str(), v[1].c_str());
		}

	      if(s1=="SPEED")
		{
		  if(s2=="Modem") ui_combobox_speed->setCurrentItem(0);
		  if(s2=="ISDN") ui_combobox_speed->setCurrentItem(1);
		  if(s2=="ISDN2x") ui_combobox_speed->setCurrentItem(2);
		  if(s2=="Cable/DSL") ui_combobox_speed->setCurrentItem(3);
		  if(s2=="T1") ui_combobox_speed->setCurrentItem(4);
		  if(s2=="T3") ui_combobox_speed->setCurrentItem(5);
		}

	      if(s1=="SEARCHSPEED")
		{
		  if(s2=="Modem") ui_combobox_searchspeed->setCurrentItem(0);
		  if(s2=="ISDN") ui_combobox_searchspeed->setCurrentItem(1);
		  if(s2=="ISDN2x") ui_combobox_searchspeed->setCurrentItem(2);
		  if(s2=="Cable/DSL") ui_combobox_searchspeed->setCurrentItem(3);
		  if(s2=="T1") ui_combobox_searchspeed->setCurrentItem(4);
		  if(s2=="T3") ui_combobox_searchspeed->setCurrentItem(5);
		}

	      if(s1=="LOGLEVEL")
		{
		  if(s2=="Debug") ui_combobox_loglevel->setCurrentItem(0);
		  if(s2=="Info") ui_combobox_loglevel->setCurrentItem(1);
		  if(s2=="Errors") ui_combobox_loglevel->setCurrentItem(2);
		}

	      if(s1=="MAXDOWNLOADS")
		{
		  std::strstream str;
		  int       max_downloads;
		  str << s2 << std::ends;
		  str >> max_downloads;
		  ui_spinbox_maxdownloads->setValue(max_downloads);
		}

 	      if(s1=="MAXUPLOADS")
		{
		  std::strstream str;
		  int       max_uploads;
		  str << s2 << std::ends;
		  str >> max_uploads;
		  ui_spinbox_maxuploads->setValue(max_uploads);
		}

 	      if(s1=="UPLOADBANDWIDTH")
		{
		  std::strstream str;
		  int       upload_bandwidth;
		  str << s2 << std::ends;
		  str >> upload_bandwidth;
		  ui_spinbox_upbandwidth->setValue(upload_bandwidth);
		}

	      if(s1=="AUTOCONNECT")
		{
		  std::vector<std::string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ';');
		  for(i=0; i<vString.size(); i++)
		    ui_listbox_autoconnect->insertItem(vString[i].c_str());
		}

	      if(s1=="GEOMETRY")
		{
		  std::vector<std::string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ',');
		  int a[4];
		  for(i = 0; i < vString.size(); i++)
		    {
		      std::strstream str;
		      str << vString[i] << std::ends;
		      str >> a[i];
		    }
		  setGeometry( a[0], a[1], a[2], a[3] );
		}

	      if(s1=="BLOCKED_HOSTS")
		{
		  std::vector<std::string> vString;
		  s.assign(s2);
		  s.split_to_vector(vString, ';');
		  for(i = 0; i < vString.size(); i++)
		    ui_listbox_blockedhosts->insertItem( vString[i].c_str() );
		}

	      if(s1=="FILENAME_LENGTH")
		{
		  std::strstream str;
		  str << s2 << std::ends;
		  str >> i;
		  ui_spinbox_filelength->setValue(i);
		}
	      
	      if(s1=="EXISTING_FILES")
		{
		  if(s2=="0") ui_radiobutton_e_overwrite->setChecked(true);
		  else
		    if(s2=="1") ui_radiobutton_e_rename->setChecked(true);
		    else
		      ui_radiobutton_e_abort->setChecked(true);
		}

	      if(s1=="ENABLE_LOGFILE") if(s2=="1") ui_checkbox_loginfile->setChecked(true);
	      
	      if(s1=="SAVE_HOST_LIST")
		{
		  if(s2=="0") ui_checkbox_savehostlist->setChecked(false);
		  else
		    ui_checkbox_savehostlist->setChecked(true);
		}

	      if(s1=="ONEXIT_REMOVE_SIZE")
		{
		  std::strstream str;
		  int            size;
		  str << s2 << std::ends;
		  str >> size;
		  ui_spinbox_conf_filesize->setValue(size);
		}

	      if(s1=="ONEXIT_REMOVE") if(s2=="1") ui_checkbox_conf_removefiles->setChecked(true);
	      if(s1=="ONEXIT_REMOVEALL") if(s2=="1") ui_checkbox_conf_removeall->setChecked(true);
	      if(s1=="IGNORE_FIREWALLED") if(s2=="1") ui_checkbox_ignorefirewalled->setChecked(true);
	      if(s1=="IGNORE_BUSY") if(s2=="1") ui_checkbox_ignorebusy->setChecked(true);
	      if(s1=="FILTER_PRIVATE_IP") if(s2=="0") ui_checkbox_filterip->setChecked(false);
	      if(s1=="ENABLE_IN_SEARCHES") if(s2=="1") ui_checkbox_incomingsearch->setChecked(true);
	      if(s1=="ENABLE_LOGS") if(s2=="1") ui_checkbox_enablelogs->setChecked(true);
	      if(s1=="ENABLE_LOGFILE") if(s2=="1") ui_checkbox_loginfile->setChecked(true);
	      if(s1=="ONSTART_CONNECT") if(s2=="1") ui_checkbox_conf_autoconnect->setChecked(true);
	      if(s1=="HOSTCACHE_ENABLE") if(s2=="1") ui_checkbox_enablehostcache->setChecked(true);
	      if(s1=="HOSTCACHE_ENABLELOG") if(s2=="0") ui_checkbox_enablelogging->setChecked(false);
	      if(s1=="HOSTCACHE_ENABLEFILELOG") if(s2=="1") ui_checkbox_hostcacheenablefilelog->setChecked(true);
	      if(s1=="FILE_ASSOCIATIONS") if(s2=="1") ui_checkbox_fileassociations->setChecked(true);
	      if(s1=="SHOW_DELETE_DIALOG") if(s2=="0") _show_delete_dialog = false;
	      if(s1=="SHOW_PROGR_BAR") if(s2=="0") ui_checkbox_showprogressbar->setChecked(false);
	      if(s1=="NUMERIC_PROGRESS") if(s2=="1") ui_checkbox_numericprogress->setChecked(true);
	      if(s1=="SHOW_UP_PROGR_BAR") if(s2=="0") ui_checkbox_up_showprogressbar->setChecked(false);
	      if(s1=="NUMERIC_UP_PROGRESS") if(s2=="1") ui_checkbox_up_progressvalue->setChecked(true);
	      if(s1=="HIDE_FILENAMES") if(s2=="1") ui_checkbox_hidefilename->setChecked(true);
	      if(s1=="HIDE_FILESIZE") if(s2=="1") ui_checkbox_hidefilesize->setChecked(true);
	      if(s1=="SORT_CASESENSITIVE") if(s2=="0") ui_checkbox_casesensitive->setChecked(false);
	      if(s1=="SEARCHES_MATCH") if(s2=="1") ui_checkbox_searches_match->setChecked(true);
	      if(s1=="SYSTEM_TRAY") if(s2=="0") ui_checkbox_systemtray->setChecked(false);
	      if(s1=="GROUP_SIZE") if(s2=="1") ui_checkbox_group_size->setChecked(true);
	      if(s1=="SHOW_ONLY_ESTABLISHED") if(s2=="1") ui_checkbox_showestablished->setChecked(true);

	      if(s1=="HOSTCACHE_PORT")
		{
		  std::strstream str;
		  int i;
		  str << s2 << std::ends;
		  str >> i;
		  ui_spinbox_hostcacheport->setValue(i);
		}

	      if(s1=="RETRIES")
		{
		  std::strstream str;
		  int            n;
		  str << s2 << std::ends;
		  str >> n;
		  ui_spinbox_conf_retries->setValue(n);
		}

	      if(s1=="INCONNECTS")
		{
		  std::strstream str;
		  int            n;
		  str << s2 << std::ends;
		  str >> n;
		  ui_spinbox_inconnects->setValue(n);
		}

	      if(s1=="DL_PER_HOST")
		{
		  std::strstream str;
		  int            n;
		  str << s2 << std::ends;
		  str >> n;
		  ui_spinbox_dl_per_host->setValue(n);
		}

	      if(s1=="IMAGES") ui_lineedit_images->setText(s2.c_str());
	      if(s1=="VIDEOS") ui_lineedit_video->setText(s2.c_str());
	      if(s1=="AUDIO") ui_lineedit_audio->setText(s2.c_str());

	      if(s1=="LOGFILE") ui_lineedit_logfile->setText(s2.c_str());

	      if(s1=="ABORT_DOWNLOADS")
		{
		  if(s2=="0") ui_radiobutton_a_contsize->setChecked(true);
		  else
		    if(s2=="1") ui_radiobutton_a_conthost->setChecked(true);
		    else
		      if(s2=="2") ui_radiobutton_a_contsizehost->setChecked(true);
		      else
			ui_radiobutton_a_startnew->setChecked(true);
		}

	      if(s1=="NHOSTS")
		{
		  std::strstream str;
		  int       hosts;
		  str << s2 << std::ends;
		  str >> hosts;
		  ui_spinbox_hosts->setValue(hosts);
		}

	      if(s1=="DELIVER_HOSTS")
		{
		  std::strstream str;
		  int       hosts;
		  str << s2 << std::ends;
		  str >> hosts;
		  ui_spinbox_deliverhosts->setValue(hosts);
		}

	      if(s1=="TTL")
		{
		  std::strstream str;
		  int    ttl;
		  str << s2 << std::ends;
		  str >> ttl;
		  ui_spinbox_ttl->setValue(ttl);
		}

	      if(s1=="IP_UPDATE_INTERVAL")
		{
		  std::strstream str;
		  int    i;
		  str << s2 << std::ends;
		  str >> i;
		  ui_spinbox_updateinterval->setValue(i);
		}

	      if(s1 == "VIEW_CONNECTIONS") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(1, false);
		    Tabular->removePage(page_connections);
		  }

	      if(s1 == "VIEW_SEARCH") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(2, false);
		    Tabular->removePage(page_search);
		  }

	      if(s1 == "VIEW_DOWNLOADS") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(3, false);
		    Tabular->removePage(page_downloads);
		  }

	      if(s1 == "VIEW_UPLOADS") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(4, false);
		    Tabular->removePage(page_uploads);
		  }

	      if(s1 == "VIEW_SHARED_FILES") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(5, false);
		    Tabular->removePage(page_shared_files);
		  }

	      if(s1 == "VIEW_LOGS") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(6, false);
		    Tabular->removePage(page_logs);
		  }

	      if(s1 == "VIEW_STATISTICS") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(7, false);
		    Tabular->removePage(page_statistics);
		  }

	      if(s1 == "VIEW_HOSTCACHE") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(8, false);
		    Tabular->removePage(page_hostcache);
		  }
		  
	      if(s1 == "VIEW_INTERRUPTED_DL") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(9, false);
		    Tabular->removePage(page_interrupted_dl);
		  }

	      if(s1 == "VIEW_CONFIGURATION") 
		if(s2 == "0") 
		  {
		    _view->setItemChecked(10, false);
		    Tabular->removePage(page_configuration);
		  }

	    }
	}
      f.close();
    }
  else  // default configuration
    {
      ui_listbox_autoconnect->insertItem("qtellaconnect.ath.cx:6347");
      ui_listbox_autoconnect->insertItem("router.limewire.com:6346");
      ui_listbox_autoconnect->insertItem("connect1.gnutellanet.com:6346");
      ui_listbox_autoconnect->insertItem("connect2.gnutellanet.com:6346");
      ui_listbox_autoconnect->insertItem("connect3.gnutellanet.com:6346");
      ui_listbox_autoconnect->insertItem("connect4.gnutellanet.com:6346");
      ui_listbox_autoconnect->insertItem("connect3.bearshare.net:6346");
      
      _prev_version = "99.0.0";
    }

  _read_config = false;
}


void QtellaSub::slotApplyConfig()
{
  SearchWidget *c = (SearchWidget*) ui_tabwidget_search->currentPage();
  if( c ) _search_column_size = c->ui_listview_search->columnWidth(2);

  QRect        r = geometry();
  std::fstream f;

  f.open(conffile.c_str(), std::ios::out | std::ios::trunc);
  if(!f.is_open())
    {
      std::cerr << "Cannot open configuration file. " << conffile << std::endl;
      return;
    }
  f << "GEOMETRY=" << r.x() << "," << r.y() << "," << r.width() << "," << r.height() << std::endl
    << "DOWNLOAD=" << (const char*)ui_lineedit_downloaddirectory->text() << std::endl
    << "INCOMPLETE=" << (const char*)ui_lineedit_incompleted->text() << std::endl
    << "PORT=" << (const char*)ui_lineedit_listenport->text() << std::endl
    << "SPEED=" << (const char*)ui_combobox_speed->currentText() << std::endl
    << "SEARCHSPEED=" << (const char*)ui_combobox_searchspeed->currentText() << std::endl
    << "LOGLEVEL=" << (const char*)ui_combobox_loglevel->currentText() << std::endl
    << "HOSTCACHE_LOGFILE=" << (const char*)ui_lineedit_hostcachelogfile->text() << std::endl
    << "HOSTCACHE_PORT=" << ui_spinbox_hostcacheport->value() << std::endl
    << "HOSTCACHE_ENABLE=" << ui_checkbox_enablehostcache->isChecked() << std::endl
    << "HOSTCACHE_ENABLELOG=" << ui_checkbox_enablelogging->isChecked() << std::endl
    << "HOSTCACHE_ENABLEFILELOG=" << ui_checkbox_hostcacheenablefilelog->isChecked() << std::endl;

  f << "MAXDOWNLOADS=" << ui_spinbox_maxdownloads->value() << std::endl
    << "MAXUPLOADS=" << ui_spinbox_maxuploads->value() << std::endl
    << "UPLOADBANDWIDTH=" << ui_spinbox_upbandwidth->value() << std::endl
    << "FILENAME_LENGTH=" << ui_spinbox_filelength->value() << std::endl
    << "SAVE_HOST_LIST=" << ui_checkbox_savehostlist->isChecked() << std::endl
    << "ONSTART_CONNECT=" << ui_checkbox_conf_autoconnect->isChecked() << std::endl
    << "ONEXIT_REMOVE=" << ui_checkbox_conf_removefiles->isChecked() << std::endl
    << "ONEXIT_REMOVEALL=" << ui_checkbox_conf_removeall->isChecked() << std::endl
    << "ONEXIT_REMOVE_SIZE=" << ui_spinbox_conf_filesize->value() << std::endl
    << "ENABLE_IN_SEARCHES=" << ui_checkbox_incomingsearch->isChecked() << std::endl
    << "ENABLE_LOGFILE=" << ui_checkbox_loginfile->isChecked() << std::endl
    << "ENABLE_LOGS=" << ui_checkbox_enablelogs->isChecked() << std::endl
    << "RETRIES=" << ui_spinbox_conf_retries->value() << std::endl
    << "IGNORE_FIREWALLED=" << ui_checkbox_ignorefirewalled->isChecked() << std::endl
    << "IGNORE_BUSY=" << ui_checkbox_ignorebusy->isChecked() << std::endl
    << "FILTER_PRIVATE_IP=" << ui_checkbox_filterip->isChecked() << std::endl;

  f << "HOST_LIST_FILE=" << (const char*)ui_lineedit_hostlistfile->text() << std::endl
    << "LOGFILE=" << (const char*)ui_lineedit_logfile->text() << std::endl
    << "IMAGES=" << (const char*)ui_lineedit_images->text() << std::endl
    << "VIDEOS=" << (const char*)ui_lineedit_video->text() << std::endl
    << "AUDIO=" << (const char*)ui_lineedit_audio->text() << std::endl
    << "NHOSTS=" << ui_spinbox_hosts->value() << std::endl
    << "TTL=" << ui_spinbox_ttl->value() << std::endl
    << "DELIVER_HOSTS=" << ui_spinbox_deliverhosts->value() << std::endl
    << "IFACE=" << (const char*)ui_combobox_interface->currentText() << std::endl
    << "LASTADDR=" << (const char*)ui_lineedit_ip->text() << std::endl
    << "INCONNECTS=" << ui_spinbox_inconnects->value() << std::endl
    << "VERSION=" << VERSION << std::endl
    << "DL_PER_HOST=" << ui_spinbox_dl_per_host->value() << std::endl
    << "FILE_ASSOCIATIONS=" << ui_checkbox_fileassociations->isChecked() << std::endl
    << "SHOW_DELETE_DIALOG=" << _show_delete_dialog << std::endl
    << "TABSIZE_DOWNLOAD=" << ui_listview_download->columnWidth(0) << std::endl
    << "TABSIZE_SHARED=" << ui_listview_shared->columnWidth(0) << std::endl
    << "TABSIZE_UPLOADS=" << ui_listview_uploads->columnWidth(0) << std::endl
    << "TABSIZE_INTERRUPT=" << ui_listview_interrupt->columnWidth(0) << std::endl
    << "TABSIZE_SEARCH=" << _search_column_size << std::endl;

  f << "SHOW_PROGR_BAR=" << ui_checkbox_showprogressbar->isChecked() << std::endl
    << "NUMERIC_PROGRESS=" << ui_checkbox_numericprogress->isChecked() << std::endl
    << "HOSTNAME=" << (const char*)ui_lineedit_hostname->text() << std::endl
    << "IP_UPDATE_INTERVAL=" << ui_spinbox_updateinterval->value() << std::endl
    << "HIDE_FILENAME=" << ui_checkbox_hidefilename->isChecked() << std::endl
    << "HIDE_FILESIZE=" << ui_checkbox_hidefilesize->isChecked() << std::endl
    << "SEARCHES_MATCH=" << ui_checkbox_searches_match->isChecked() << std::endl
    << "SORT_CASESENSITIVE=" << ui_checkbox_casesensitive->isChecked() << std::endl
    << "SHOW_UP_PROGR_BAR=" << ui_checkbox_up_showprogressbar->isChecked() << std::endl
    << "NUMERIC_UP_PROGRESS=" << ui_checkbox_up_progressvalue->isChecked() << std::endl
    << "GROUP_SIZE=" << ui_checkbox_group_size->isChecked() << std::endl
    << "SYSTEM_TRAY=" << ui_checkbox_systemtray->isChecked() << std::endl
    << "VIEW_CONNECTIONS=" << _view->isItemChecked(1) << std::endl
    << "VIEW_SEARCH=" << _view->isItemChecked(2) << std::endl
    << "SHOW_ONLY_ESTABLISHED=" << ui_checkbox_showestablished->isChecked() << std::endl;

  f << "VIEW_DOWNLOADS=" << _view->isItemChecked(3) << std::endl
    << "VIEW_UPLOADS=" << _view->isItemChecked(4) << std::endl
    << "VIEW_SHARED_FILES=" << _view->isItemChecked(5) << std::endl
    << "VIEW_LOGS=" << _view->isItemChecked(6) << std::endl
    << "VIEW_STATISTICS=" << _view->isItemChecked(7) << std::endl
    << "VIEW_HOSTCACHE=" << _view->isItemChecked(8) << std::endl
    << "VIEW_INTERRUPTED_DL=" << _view->isItemChecked(9) << std::endl
    << "VIEW_CONFIGURATION=" <<  _view->isItemChecked(10)<< std::endl
    << "STYLE=" << _style_s << std::endl
    << "LAST_DL_TO_DIR=" << _last_download_to_dir << std::endl
    << "SHAREDDIR=";

  int i;

  for(i = 0; i < ui_listbox_shareddirectories->count(); i++)
    f << (const char*)ui_listbox_shareddirectories->text(i) << ";";

  f << std::endl;

  for(i = 0; i < _applications.size(); ++i)
    f << "APPS=" << (_applications[i].first + ":" + _applications[i].second) << std::endl;

  f << "AUTOCONNECT=";
  for(i = 0; i < ui_listbox_autoconnect->count(); i++)
    f << (const char*)ui_listbox_autoconnect->text(i) << ";";
  f << std::endl;

  f << "EXISTING_FILES=";
  if(ui_radiobutton_e_overwrite->isChecked()) f << "0";
  if(ui_radiobutton_e_rename->isChecked()) f << "1";
  if(ui_radiobutton_e_abort->isChecked()) f << "2";
  f << std::endl;

  f << "ABORT_DOWNLOADS=";
  if(ui_radiobutton_a_contsize->isChecked()) f << "0";
  if(ui_radiobutton_a_conthost->isChecked()) f << "1";
  if(ui_radiobutton_a_contsizehost->isChecked()) f << "2";
  if(ui_radiobutton_a_startnew->isChecked()) f << "3";
  f << std::endl;

  f << "BLOCKED_HOSTS=";
  for(i = 0; i < ui_listbox_blockedhosts->count(); ++i)
    f << (const char*)ui_listbox_blockedhosts->text(i) << ";";
  f << std::endl;

  f.close();
}


const void QtellaSub::setReportedIP(const std::string& ip)
{
  std::string iface(ui_combobox_interface->currentText());

#ifdef _DEBUG
  qDebug( "QtellaSub:: reported IP: <%s>", ip.c_str() );
#endif

  if( iface == std::string( tr("automatic", "network interface") ) )
    {
      _last_addr = ip;
      slotIPUpdate();
    }
}
