#include "QtellaSub.h"
#include "Info.h"
#include "Connections.h"
#include "InterruptedDownloads.h"

#if QT_VERSION < 300
#include <qplatinumstyle.h>
#include <qwindowsstyle.h>
#include <qmotifstyle.h>
#include <qmotifplusstyle.h>
#include <qcdestyle.h>
#include <qsgistyle.h>
#else
#include <qstylefactory.h>
#endif

#ifdef KDE
#if QT_VERSION < 300      /* patch by N.Cat 2002/03/07 */
#include "kdestyle.h" 
#endif
#endif

#include "qfont.h"
#include "qfontdialog.h"
#include "qwidget.h"
#include "qstring.h"
#include "qtabwidget.h"
#include "qtimer.h"
#include "qlabel.h"
#include "qpopupmenu.h"

void QtellaSub::slotStyle(int st)
{
#if QT_VERSION >= 300
  QStyle *style=QStyleFactory::create(_style->text(st).mid(1));
  if(style)
    qApp->setStyle(style);
#endif
}

void QtellaSub::slotStylePlatinum()
{
#if QT_VERSION < 300
  qApp->setStyle( new QPlatinumStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(0, true);
  _style_s = "PLATINUM";
#endif
}

void QtellaSub::slotStyleWindows()
{
#if QT_VERSION < 300
  qApp->setStyle( new QWindowsStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(2, true);
  _style_s = "WINDOWS";
#endif
}

void QtellaSub::slotStyleMotif()
{
#if QT_VERSION < 300
  qApp->setStyle( new QMotifStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(3, true);
  _style_s = "MOTIF";
#endif
}

void QtellaSub::slotStyleMotifPlus()
{
#if QT_VERSION < 300
  qApp->setStyle( new QMotifPlusStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(4, true);
  _style_s = "MOTIFPLUS";  
#endif
}

void QtellaSub::slotStyleCDE()
{
#if QT_VERSION < 300
  qApp->setStyle( new QCDEStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(5, true);
  _style_s = "CDE";
#endif
}

void QtellaSub::slotStyleSGI()
{
#if QT_VERSION < 300
  qApp->setStyle( new QSGIStyle() );
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(1, true);
  _style_s = "SGI";
#endif
}

void QtellaSub::slotStyleKDE()
{
#if QT_VERSION < 300
#ifdef KDE
  qApp->setStyle( new KDEStyle() );
  qApp->setPalette(_palette, true);
  menu->setPalette(_menu_palette, true);
  for(int i = 0; i < 7; ++i) _style->setItemChecked(i, false);
  _style->setItemChecked(6, true);
  _style_s = "KDE";
#endif
#endif
}

void QtellaSub::slotChangeFont()
{
  bool ok;

  QFont font = QFontDialog::getFont(&ok, _current_font, this);

  if(ok)
    {
      _current_font = font;
      qApp->setFont(font, true);
    }
}

void QtellaSub::slotViewChanged(int id)
{
  QWidget* a[10] = { 
    page_connections, 
    page_search, 
    page_downloads, 
    page_uploads,
    page_shared_files, 
    page_logs, 
    page_statistics, 
    page_hostcache,
    page_interrupted_dl, 
    page_configuration 
  };

  QString titles[10] = {
    tr("Connections"),
    tr("Search"),
    tr("Downloads"),
    tr("Uploads"),
    tr("Shared Files"),
    tr("Logs"),
    tr("Statistics"),
    tr("HostCache"),
    tr("Interrupted DL"),
    tr("Configuration")
  };

  if( _view->isItemChecked(id) )
    {
      _view->setItemChecked(id, false);
      Tabular->removePage(a[id-1]);
    }
  else
    {
      int position = id - 1;

      for(int i = 1; i < id; ++i)
	if( ! _view->isItemChecked(i) ) --position;

      _view->setItemChecked(id, true);
      Tabular->insertTab(a[id-1], titles[id-1], position);
    }
}

void QtellaSub::slotConnect()
{
  QtellaSub::getInstance()->_incoming_allowed = true;

  if( ! timer_connection->isActive() )
    {
      connections->start();
      timer_connection->start(1000);
    }
}


void QtellaSub::slotDisconnect()
{
  QtellaSub::getInstance()->_incoming_allowed = false;

  if( timer_connection->isActive() )
    {
      timer_connection->stop();
      connections->disconnect();
    }
}

void QtellaSub::slotConvertFilenames()
{
  _interrupted_downloads->convertFilenames();
}

void QtellaSub::slotInfo()
{
  Info i(this, tr("Info"), true);
  i.ui_textlabel_version->setText(VERSION);
  i.exec();
}
