#include "Gnutella.h"
#include "QueryHit.h"
#include "Errors.h"
#include "Message.h"

#include <cctype>

QueryHit::QueryHit(const Q_UINT8 ttl, const Q_UINT8 hops, const std::string id, const std::string data)
  : _ttl(ttl), _hops(hops), _id(id), _valid(true)
{
  if( data.size() < 11 ) 
    { 
      qDebug("invalid initial size");
      toString(); 
      _valid = false;
      return;
    }

  std::string qh(data.data(), data.size());

  Q_UINT16 port  = Message::str_uint16(qh.substr(1, 2).data());
  Q_UINT32 ip    = Message::str_uint32(qh.substr(3, 4).data());

  _address = Address(ip, port);
  _hits  = *((Q_UINT8*)qh.substr(0, 1).data());
  _speed = Message::str_uint32(qh.substr(7, 4).data());

  qh.erase(0, 11);

  if( qh.size() < 16 )
    {
      qDebug("error");
      toString();
      _valid = false;
      return;
    }

  _id_result = qh.substr(qh.size() - 16);
  qh.erase(qh.size() - 16, 16);

  std::string::size_type idx1;
  std::string::size_type idx2;

  for(int hits_counter = 0; hits_counter < _hits; hits_counter++)
    {
      Q_UINT32 index = Message::str_uint32(qh.substr(0, 4).data());
      Q_UINT32 fsize = Message::str_uint32(qh.substr(4, 4).data());

      qh.erase(0, 8);

      idx1 = qh.find( std::string("\0", 1) );
      
      if(idx1 == std::string::npos) // delimiter not found => clear rest of query hit
	{
	  qh.erase();
	  break;
	}

      idx2 = qh.find( std::string("\0", 1), idx1 + 1 );

      if(idx2 == std::string::npos) // delimiter not found => clear rest of query hit
	{
	  qh.erase();
	  break;
	}

      std::string fname = qh.substr(0, idx1);

      std::string additional;

      if(idx2 > (idx1 + 1))
	{
	  std::string s = qh.substr(idx1+1, idx2-idx1-1);
	  std::string::size_type idx = s.find(char(28));
	  if( idx != std::string::npos ) s = s.substr(0, idx);
	  additional = s;

	  std::string my_fname = fname;
	  for(int i = 0; i < fname.size(); ++i) fname[i] = tolower(fname[i]);
	  if( my_fname.find(".mp3") == std::string::npos ) additional = "";
	}

      qh.erase(0, idx2 + 1);
      _results.push_back(QueryResult(index, fsize, fname, additional));
    }


  if(qh.size() > 0)  // extended query hit
    {
      _trailer = qh;
      //cout << _trailer << endl;
    }
  else
    _trailer = "";

  toString();
}



QueryHit::QueryHit(const std::string header_id, const Q_UINT8 ttl, const Q_UINT8 hops, Address& address, const Q_UINT32 speed, const std::string result_id, std::vector<QueryResult>& results)
{
  _id = header_id;
  _ttl = ttl;
  _hops = hops;
  _hits = results.size();
  _address = address;
  _speed = speed;
  _id_result = result_id;
  _results = results;
  _trailer.assign( std::string("QTEL\2\0\0", 7) );
  _valid = true;

  toString();
}

void QueryHit::toString()
{
  _data = _id;

  _data += static_cast<unsigned char>(CMD_QUERYHIT);
  _data += static_cast<unsigned char>(_ttl);
  _data += static_cast<unsigned char>(_hops);

  std::string result;

  for(int i = 0; i < _results.size(); ++i)
    {
      result.append( Message::uint_str(_results[i]._index) );
      result.append( Message::uint_str(_results[i]._filesize) );
      result.append( _results[i]._filename );
      result.append( std::string("\0\0", 2) );
    }

  result.append(_trailer);
  result.append(_id_result);

  Q_UINT32 payload = result.size() + 1+2+4+4;
  Q_UINT16 port = _address.port();
  Q_UINT32 ip = _address.longIP();

  _data.append( Message::uint_str(payload) );

  _data.append( Message::uint_str(_hits) );
  _data.append( Message::uint_str(port) );
  _data.append( Message::uint_str(ip) );
  _data.append( Message::uint_str(_speed) );

  _data.append( result );
}


bool QueryHit::isValid()
{
  return _valid;
}
