#include "SearchViewItem.h"
#include "Connections.h"
#include "QueryHitEntry.h"
#include "QtellaSub.h"
#include "Search.h"
#include "SearchWidget.h"

#include <qlistview.h>
#include <qstring.h>
#include <qcheckbox.h>

SearchViewItem::SearchViewItem(QListViewItem *parent, Connections* connections, QListView *search_list)
  : QListViewItem(parent, NULL, ""), _connections(connections), _parent(search_list), _group_parent(false)
{
}

SearchViewItem::SearchViewItem(QListView *parent, Connections* connections)
  : QListViewItem(parent), _connections(connections), _parent(parent), _group_parent(true)
{
}


SearchViewItem::SearchViewItem(QListView *parent, Connections* connections, QListViewItem *after)
  : QListViewItem(parent, after), _connections(connections), _parent(parent), _group_parent(true)
{
}

QString SearchViewItem::key(int column, bool ascending) const
{
  switch(column)
    {
    case 2:  // filename
      if( QtellaSub::getInstance()->ui_checkbox_casesensitive->isChecked() )
	return text(column); // if sorting is case-sensitive just return the string as it is
      else
	return text(column).lower();
	  
    case 4:  // filesize
    case 0:  // hit number
      return text(column).rightJustify(15, '0') + text(0).rightJustify(10, '0');
      break;

    case 1:  // status
      int  i = 0;
      int  j = 0;
      int  n_search = _connections->_search.size();
      bool found = false;

      if(!n_search) break;

      for(i = 0; i < n_search; ++i)
	if(_connections->_search[i]->sw->ui_listview_search == _parent)
	  {
	    for(j = 0; j < _connections->_search[i]->vSearchResults.size(); ++j)
	      if(_connections->_search[i]->vSearchResults[j]->item == (QListViewItem*)this ||
		 _connections->_search[i]->vSearchResults[j]->item == NULL )
		{
		  found = true;
		  break;
		}
	    break;
	  }

      if(found)
	{
	  Q_UINT32 speed  = _connections->_search[i]->vSearchResults[j]->speed;
	  int      status = _connections->_search[i]->vSearchResults[j]->status;

	  std::string s_status;

	  if(status & QueryHitEntry::Ready) s_status = "A";
	  if(status & QueryHitEntry::Unknown) s_status = "C";
	  if(status & QueryHitEntry::Firewalled) s_status = "B";
	  if(status & QueryHitEntry::Busy) s_status = "D";
	  
	  std::strstream str;
	  str << (999999-speed) << std::ends;
	  
	  std::string s(str.str());
	  str.freeze(false);
	  while(s.size() < 10) s.insert(0, "9");
	  
	  QString ret( (s_status+s).c_str() );
	  return(ret + text(0).rightJustify(5, '0'));
	}
    }

  return text(column);
}
